// This code implementation is the intellectual property of
// the GEANT4 collaboration.
//
// By copying, distributing or modifying the Program (or any work
// based on the Program) you indicate your acceptance of this statement,
// and all its terms.
//
// $Id: MulanDetectorConstruction.hh,v 1.6 2004/02/03 21:08:30 asharp Exp $
// GEANT4 tag $Name:  $
//
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo..... 
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo..... 

#ifndef MulanDetectorConstruction_h
#define MulanDetectorConstruction_h 1

#include "globals.hh"
#include "PhysicalConstants.h"
#include "G4Material.hh"
#include "G4Element.hh"
#include "G4VUserDetectorConstruction.hh"
#include "MulanMagneticField.hh"
#include "G4RotationMatrix.hh"
#include "G4UniformMagField.hh"
#include "G4FieldManager.hh"
#include "G4VisAttributes.hh"

class G4Box;
class G4Trd;
class G4Tubs;
class G4Trap;
class G4Sphere;
class G4Polyhedra;
class G4Polycone;
class G4LogicalVolume;
class G4VPhysicalVolume;
class G4Material;
class MulanDetectorMessenger;

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo..... 

class MulanDetectorConstruction : public G4VUserDetectorConstruction
{
public:
  
  MulanDetectorConstruction();
  ~MulanDetectorConstruction();

public:
  
  G4VPhysicalVolume* Construct();
     
  const 
  G4double GetTargetLength()      {return targetThick;};
  G4double GetWorldRadius()       {return WorldRadius;}; 
  G4double GetHeBagLength()       {return fHeBagLength;};
  G4double GetTileThickness()     {return thicknessInTile;};
  G4double GetSHLength()          {return depthSH;};
  G4double GetBeamLineLength()    {return lengthBeamLine;};
  G4double GetBeamWindowLength()  {return lengthBeamWindow;};
  G4double GetBeamToCenter()      {return beamToCenter;};
  
  void SetTargetMaterial(G4String);
  void EchoTargetMaterial();  
  void SetMagField(G4double);
  inline void EchoMagField();
  void SetWorldMaterial(G4String);
  void EchoWorldMaterial();  
  void UpdateGeometry();
  
private:
  void buildMaterials();
  G4ThreeVector getVertex(int);
  
  G4int getHexIndex(int,int);
  G4ThreeVector hexPosition(int);
  G4RotationMatrix* hexRM(int);
  G4ThreeVector hexWallPosition(int,int);
  G4RotationMatrix* hexWallRM(int,int);
  void setHexTileParameters(int);
  G4ThreeVector hexTilePosition(int,int);
  G4RotationMatrix* hexTileRM(int,int);
  void setHexLGParameters(int,int);
  G4ThreeVector hexLGTranslate(int);
  G4RotationMatrix* hexLGRotate(int);
  G4ThreeVector hexLGPosition(int,int);
  G4RotationMatrix* hexLGRM(int,int);
  G4double getHexTileBevel();
  
  G4int getPentIndex(int,int);
  G4ThreeVector pentPosition(int);
  G4RotationMatrix* pentRM(int);
  G4ThreeVector pentWallPosition(int,int);
  G4RotationMatrix* pentWallRM(int,int);
  void setPentTileParameters(int);
  G4ThreeVector pentTilePosition(int,int);
  G4RotationMatrix* pentTileRM(int,int);
  void setPentLGParameters(int,int);
  G4ThreeVector pentLGTranslate(int);
  G4RotationMatrix* pentLGRotate(int);
  G4ThreeVector pentLGPosition(int,int);
  G4RotationMatrix* pentLGRM(int,int);
  G4double getPentTileBevel();
  
  G4Sphere*             solidWorld;    //pointer to the solid envelope 
  G4LogicalVolume*   logicWorld;    //pointer to the logical envelope
  G4VPhysicalVolume* physiWorld;    //pointer to the physical envelope
  G4Material*        WorldMater;    //pointer to the envelope material
  
  G4Box*             solidAK3Target;
  G4Tubs*            solidTarget;   //pointer to the solid Target
  G4LogicalVolume*   logicTarget;   //pointer to the logical Target
  G4VPhysicalVolume* physiTarget;   //pointer to the physical Target
  G4Material*        TargetMater;  //pointer to the Target  material
  
  G4Sphere*          solidSphere;   //pointer to the solid Sphere
  G4LogicalVolume*   logicSphere;   //pointer to the logical Sphere
  G4VPhysicalVolume* physiSphere;   //pointer to the physical Sphere
  
  G4Polyhedra*       solidSH;       //pointer to the solid super hexagon
  G4LogicalVolume*   logicSH[20];       //pointer to the logical super hexagon
  G4VPhysicalVolume* physiSH[20];       //pointer to the physical super hexagon
  G4Material*        SHMater;       //pointer to the super hexagon material
  
  G4Polyhedra*       solidSP;       //pointer to the solid super pentagon
  G4LogicalVolume*   logicSP[12];       //pointer to the logical super pentagon
  G4VPhysicalVolume* physiSP[12];       //pointer to the physical super pentagon
  G4Material*        SPMater;       //pointer to the super pentagon material
  
  G4Trap*            solidInTile;       //pointer to the solid inner detector
  G4LogicalVolume*   logicInTile[20][6];       //pointer to the logical inner detector
  G4VPhysicalVolume* physiInTile[20][6];//pointer to the physical inner detector
  G4Trap*             solidSPInTile;       //pointer to the solid inner detector
  G4LogicalVolume*   logicSPInTile[12][5];       //pointer to the logical inner detector
  G4VPhysicalVolume* physiSPInTile[12][5];//pointer to the physical inner detector
  G4Material*        inTileMater;       //pointer to the inner detector material
  G4Trd*             solidSHInLG;       //pointer to the solid inner detector
  G4LogicalVolume*   logicSHInLG[20][6];       //pointer to the logical inner detector
  G4VPhysicalVolume* physiSHInLG[20][6];//pointer to the physical inner detector
  G4Trd*             solidSPInLG;       //pointer to the solid inner detector
  G4LogicalVolume*   logicSPInLG[12][5];       //pointer to the logical inner detector
  G4VPhysicalVolume* physiSPInLG[12][5];//pointer to the physical inner detector
  G4Material*        LGMater;       //pointer to the inner detector material
  
  
  G4Trap*            solidOutTile;       //pointer to the solid inner detector
  G4LogicalVolume*   logicOutTile[20][6];       //pointer to the logical inner detector
  G4VPhysicalVolume* physiOutTile[20][6];//pointer to the physical inner detector
  G4Trap*             solidSPOutTile;       //pointer to the solid inner detector
  G4LogicalVolume*   logicSPOutTile[12][5];       //pointer to the logical inner detector
  G4VPhysicalVolume* physiSPOutTile[12][5];//pointer to the physical inner detector
  G4Material*        outTileMater;       //pointer to the inner detector material
  G4Trd*             solidSHOutLG;       //pointer to the solid inner detector
  G4LogicalVolume*   logicSHOutLG[20][6];       //pointer to the logical inner detector
  G4VPhysicalVolume* physiSHOutLG[20][6];//pointer to the physical inner detector
  G4Trd*             solidSPOutLG;       //pointer to the solid inner detector
  G4LogicalVolume*   logicSPOutLG[12][5];       //pointer to the logical inner detector
  G4VPhysicalVolume* physiSPOutLG[12][5];//pointer to the physical inner detector
  
  G4Tubs* solidBag;
  G4LogicalVolume* logicBag;
  G4VPhysicalVolume* physiBag;
  //  void PrintPhysicalVolumeContents(G4PhysicalVolume);
  //  G4Box* solidOuterWindow;
  //  G4LogicalVolume* logicOuterWindow;
  //  G4VPhysicalVolume* physiDownstreamOuterWindow;
  //  G4VPhysicalVolume* physiUpstreamOuterWindow;
  
  MulanMagneticField* fpMagField;   //pointer to the magnetic field 
  
  MulanDetectorMessenger* detectorMessenger;  //pointer to the Messenger
       
  G4double baseTile;                // Length of a tile element
  G4double thicknessInTile;         // Thickness of inner tile element
  G4double thicknessOutTile;        // Thickness of outer tile element
  G4double spacingTile;             // Spacing between tile elements
  G4double spaceTile;               // Space between inner and outer
  G4double InMinusOut;
  
  G4double widthSH;                 // Width of a super hexagons frame
  G4double depthSH;                 // Depth of a super hexagon
  G4double baseSH;                  // Length of a super hexagon
  G4double radiusSH;                // Radius of circle inscribing SH
  G4double radiusSP;                // Radius of circle inscribing SP
  G4double domeRadius;              // Radius of the dome inscribing the SH's
  G4double domeSPRadius;            // Radius of dome inscribing the SP's
  
  G4double WorldRadius;             // Outer Radius of the world volume
  G4double fTargetLength;           // Full length of Target
  G4double targetSize;              // Target radius
  G4double targetThick;             // Target thickness
  G4double fHeBagLength;            // Full length of Helium Bag
  
  G4double lengthBeamLine;		// Full length of beam line
  G4double radiusBeamLine;		// radius of the beam line
  G4double beamToCenter;		// distance from center of Mulan to beam Line
  G4Material* BeamLineMater;
  G4double lengthBeamWindow;		// Full length (width) of beam window
  G4Material* BeamWindowMater; 
  
  G4double wallThickness;
  G4double wallHeight;
  G4double wallLengthSH;
  G4double wallLengthSP;
  G4Material* wallMater;
  
  //define beam pipe parameters here
  
  
  G4double lengthLG,baseLG,topLG,thicknessLG,offsetLG;
  
  G4double fMagField;
  G4double lengthWireChamber;
  G4double baseWireChamber;
  G4Material* WireChamberMater;
  G4double lengthChamber;
  G4Material* ChamberMater;
  
  G4double hexPsi;
  G4double hexTheta;
  G4double pentPsi;
  G4double pentTheta;

  // G4Trap parameters
  G4double pDz,pTheta,pPhi,pDy1,pDx1,pDx2,pAlpha1,pDy2,pDx3,pDx4,pAlpha2; 


  //
  // Elements, defined by Z and atomic mass
  //
  G4Element *elH;   // Hydrogen,  atomic no. 1
  G4Element *elC;   // Carbon,    atomic no. 6
  G4Element *elN;   // Nitrogen,  atomic no. 7
  G4Element *elO;   // Oxygen,    atomic no. 8
  G4Element *elF;   // Flourine,  atomic no. 9
  G4Element *elSi;  // Silicon,   atomic no. 14
  G4Element *elS;   // Sulfur,    atomic no. 16
  G4Element *elCl;  // Chlorine,  atomic no. 17
  G4Element *elCr;  // Chromium,  atomic no. 24
  G4Element *elMn;  // Manganese, atomic no. 25
  G4Element *elFe;  // Iron,      atomic no. 26
  G4Element *elCo;  // Cobalt,    atomic no. 27
  G4Element *elNi;  // Nickel,    atomic no. 28
  G4Element *elSn;  // Tin,       atomic no. 50
  G4Element *elW;   // Tungsten,  atomic no. 74
  G4Element *elAu;  // Gold,      atomic no. 79

  //
  // Materials, defined by density, number of
  //            constituent elements, mass ratio, etc.
  // Bulk Solids
  G4Material *Aluminium;
  G4Material *Arnokrome3;
  G4Material *Arnokrome4; 
  G4Material *BC404;
  G4Material *BC408;
  G4Material *CarbonFiber;
  G4Material *Cobalt;
  G4Material *Gold;
  G4Material *Iron;
  G4Material *Lead;
  G4Material *Mylar;
  G4Material *Silver;
  G4Material *SSteel;
  G4Material *Sulfur;
  G4Material *Tin;
  G4Material *Tungsten;

  // Gases
  G4Material *Air;
  G4Material *ArgonGas;
  G4Material *FreonGas;
  G4Material *HeliumGas;
  G4Material *MethaneGas;
  G4Material *Vacuum;
  G4Material *WireChamberGas;
  G4Material *XenonGas;
  
public: 

  // Visualization Attributes
  G4VisAttributes *whiteSolid;
  G4VisAttributes *blackSolid;
  G4VisAttributes *redSolid;
  G4VisAttributes *greenSolid;
  G4VisAttributes *blueSolid;
  G4VisAttributes *magentaSolid;
  G4VisAttributes *cyanSolid;
  G4VisAttributes *yellowSolid;
  G4VisAttributes *whiteFrame;
  G4VisAttributes *redFrame;
  G4VisAttributes *greenFrame;
  G4VisAttributes *blueFrame;
  G4VisAttributes *magentaFrame;
  G4VisAttributes *cyanFrame;
  G4VisAttributes *yellowFrame;
  G4VisAttributes *offAtt;
  G4VisAttributes *TargetVisAtt;


};

#endif

