// This code implementation is the intellectual property of
// the GEANT4 collaboration.
//
// By copying, distributing or modifying the Program (or any work
// based on the Program) you indicate your acceptance of this statement,
// and all its terms.
//
// $Id: MulanDetectorSD.hh,v 1.20 2004/03/25 16:47:01 asharp Exp $
// GEANT4 tag $Name:  $
//
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

#ifndef MulanDetectorSD_h
#define MulanDetectorSD_h 1

#include "G4VSensitiveDetector.hh"
#include "G4DecayTable.hh"
#include "MulanDetectorHit.hh"
#include "TROOT.h"
#include "TH1.h"
#include "TH2.h"
#include "TTree.h"
#include "TFile.h"
#include <fstream.h>
#include "TObject.h"           // from ROOT libraries
#include "TReadoutBranch.hh"   // from mulan/include
#include "TReadoutWorld.hh"    // from mulan/include
#include "TReadoutTarget.hh"   // from mulan/include
#include "TReadoutTilePair.hh" // from mulan/include
#include "TReadoutSimple.hh"   // from mulan/include

class G4Step;
class G4HCofThisEvent;

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

class MulanDetectorSD : public G4VSensitiveDetector
{

public:
  MulanDetectorSD(G4String);
  ~MulanDetectorSD();

  void Initialize(G4HCofThisEvent*);
  G4bool ProcessHits(G4Step*, G4TouchableHistory*);
  void EndOfEvent(G4HCofThisEvent*);
  void ProcessEventHC(G4HCofThisEvent*);

  inline G4int FindPartner(G4int serialNum) {
    G4int partnerSerialNum = 0;
    if(serialNum%2 == 0) partnerSerialNum = (serialNum-1);
    else partnerSerialNum = (serialNum+1);
    return partnerSerialNum;
  }

private:
  //////////////////////////////////
  // GEANT4 objects and variables //
  //////////////////////////////////
  ofstream newInfo;
  MulanDetectorHitsCollection* detectorCollection;
  G4bool EventPreparedToPrint;

  G4int parentID,T_ID,badFirstStep,hitS_ID,hitT_ID;
  G4double thisGlobalTime, thisLocalTime;

  G4String particleName;
  G4VPhysicalVolume* currentVolume;
  G4String volumeName;
  G4VPhysicalVolume* nextVolume;
  G4int ParticleType;
  inline G4double FindEarliestTime()  {return 0;}
  G4double totalEdep;
  G4double singleEdep;
  G4int tilesHit;
  G4int tilesHitECut;
  G4int serialNum;
//    G4int partnerSerialNum;
  G4double hitTime;
  G4int counter;
  G4int evtIndex;
  G4bool Prim4VecSet;
  G4bool Decay4VecSet;
  G4bool DecayVolSet;
  //////////////////
  // ROOT objects //
  //////////////////
  TFile* rootFile;
  TH1D*  TotalEdepPerEvent;
  TH1D*  NumTilesHit;
  TH1D*  NumTilesHitEdepCut;
  TH2D*  EdepPerTilePair;
  TTree* DataTree;
  TReadoutWorld*  worldBranch;
  TReadoutTarget* targetBranch;
  TReadoutTilePair* tileBranch1;
  TReadoutTilePair* tileBranch2;
  TReadoutTilePair* tileBranch3;
  TReadoutTilePair* tileBranch4;

public:
  G4int fixHexHouseID(int id);
  G4int fixHexTileID(int id, int tile);
  G4int fixPentHouseID(int id);
  G4int fixPentTileID(int id, int tile);
  G4int GetZipcode(G4String, G4int, G4int);
  G4int GetSerialNumber(int zipcode);
  G4int GetParticleType(G4String particle, G4String process);
  G4int GetParticleType(G4String particle, G4int parentID);

};

#endif





