#ifndef MulanTrackInformation_h
#define MulanTrackInformation_h 1

#include "globals.hh"
#include "G4ThreeVector.hh"
#include "G4ParticleDefinition.hh"
#include "G4Track.hh"
#include "G4Allocator.hh"
#include "G4VUserTrackInformation.hh"

class MulanTrackInformation : public G4VUserTrackInformation 
{
  public:
    MulanTrackInformation();
    MulanTrackInformation(const G4Track* aTrack);
    MulanTrackInformation(const MulanTrackInformation* aTrackInfo);
    virtual ~MulanTrackInformation();
   
    inline void *operator new(size_t);
    inline void operator delete(void *aTrackInfo);
    inline int operator ==(const MulanTrackInformation& right) const
    {return (this==&right);}

    void Print() const;

  private:
    G4int                 originalTrackID;
    G4ParticleDefinition* particleDefinition;
    G4ThreeVector         originalPosition;
    G4ThreeVector         originalMomentum;
    G4double              originalEnergy;
    G4double              originalTime;

  public:
    inline G4int GetOriginalTrackID() const {return originalTrackID;}
    inline G4ParticleDefinition* GetOriginalParticle() const {return particleDefinition;}
    inline G4ThreeVector GetOriginalPosition() const {return originalPosition;}
    inline G4ThreeVector GetOriginalMomentum() const {return originalMomentum;}
    inline G4double GetOriginalEnergy() const {return originalEnergy;}
    inline G4double GetOriginalTime() const {return originalTime;}
  inline void SaySomething() {G4cout<<"this is info "<<G4endl; return;}
};

extern G4Allocator<MulanTrackInformation> aTrackInformationAllocator;

inline void* MulanTrackInformation::operator new(size_t)
{ void* aTrackInfo;
  aTrackInfo = (void*)aTrackInformationAllocator.MallocSingle();
  return aTrackInfo;
}

inline void MulanTrackInformation::operator delete(void *aTrackInfo)
{ aTrackInformationAllocator.FreeSingle((MulanTrackInformation*)aTrackInfo);}

#endif
