// This code implementation is the intellectual property of
// the GEANT4 collaboration.
//
// By copying, distributing or modifying the Program (or any work
// based on the Program) you indicate your acceptance of this statement,
// and all its terms.
//
// $Id: MulanVolumeStepCut.hh,v 1.1.1.1 2003/10/24 19:10:20 asharp Exp $
// GEANT4 tag $Name:  $
//

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

#ifndef MulanVolumeStepCut_h
#define MulanVolumeStepCut_h 1

#include "G4ios.hh"
#include "globals.hh"
#include "G4VDiscreteProcess.hh"
#include "G4Step.hh"
#include "MulanDetectorConstruction.hh"

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

class MulanVolumeStepCut : public G4VDiscreteProcess
{
  public:     

     MulanVolumeStepCut(MulanDetectorConstruction* ,
                        const G4String& processName ="UserStepCut");
     MulanVolumeStepCut(MulanVolumeStepCut &);

    ~MulanVolumeStepCut();

     void SetMaxStep(G4double step)		{fMaxStep = step;};

     void SetScaleFactor(G4double scale)	{fScaleFactor = scale;};

     void SetTargetStep(G4double step)		{fTargetStep = step;};

     void SetHeBagStep(G4double step)		{fHeBagStep = step;};

     void SetWorldStep(G4double step)		{fWorldStep = step;};

     void SetTileStep(G4double step)		{fTileStep = step;};

     void SetSuperTriangleStep(G4double step)	{fSuperTriangleStep = step;};

     void SetBeamLineStep(G4double step)	{fBeamLineStep = step;};

     void SetBeamWindowStep(G4double step)	{fBeamWindowStep = step;};

     void SetTargetScale(G4double scale)	{fTargetScale = scale;};

     void SetHeBagScale(G4double scale)		{fHeBagScale = scale;};

     void SetWorldScale(G4double scale)		{fWorldScale = scale;};

     void SetTileScale(G4double scale)		{fTileScale = scale;};

     void SetSuperTriangleScale(G4double scale)	{fSuperTriangleScale = scale;};

     void SetBeamLineScale(G4double scale)	{fBeamLineScale = scale;};

     void SetBeamWindowScale(G4double scale)	{fBeamWindowScale = scale;};


     G4double PostStepGetPhysicalInteractionLength(
                             const G4Track& track,
			     G4double   previousStepSize,
			     G4ForceCondition* condition
			    );

     G4VParticleChange* PostStepDoIt(const G4Track&, const G4Step&);


  protected:

     // it is not needed here !
     G4double GetMeanFreePath(const G4Track& aTrack,
                             G4double   previousStepSize,
                             G4ForceCondition* condition
                            );

			    
  private:
  
    // hide assignment operator as private 
    MulanVolumeStepCut& operator=(const MulanVolumeStepCut &right);

  private:

  G4double fMaxStep,fScaleFactor;
  G4double fTargetStep,fHeBagStep, fWorldStep,  
    fTileStep, fSuperTriangleStep;    
  G4double fTargetScale,fHeBagScale, fWorldScale, 
       fTileScale, fSuperTriangleScale;    
  G4double fBeamLineStep, fBeamLineScale, fBeamWindowStep, fBeamWindowScale;
  G4double fTargetLength, fHeBagLength, fWorldRadius, fBeamLineLength,
    fBeamWindowLength, fTileLength, fSuperTriangleLength;
     
};

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....


#endif

