#ifndef READOUT_TARGET_HH
#define READOUT_TARGET_HH

#include "TVector3.h"
#include "TLorentzVector.h"
#include "TReadoutBranch.hh"

#include <iostream>
using namespace std;

class TReadoutTarget : public TReadoutBranch {

public:
  TReadoutTarget();
  //  ~TReadoutTarget();
  // the shared library won't build correctly if destructor is included!
private:
  TLorentzVector fIncidentMuon4Pos;
  TLorentzVector fIncidentMuon4Mom;
  Double_t       fMichelEdep;

public:
  TLorentzVector GetIncMuon4Pos(); // muon pos 4-vec before target hit
  TVector3       GetIncMuon3Pos(); // muon pos 3-vec before target hit
  Double_t       GetIncMuonTime(); // global time before target hit
  TLorentzVector GetIncMuon4Mom(); // muon mom 4-vec before target hit
  TVector3       GetIncMuon3Mom(); // muon mom 3-vec before target hit
  Double_t       GetIncMuonEnergy(); // muon [kin] energy b4 target hit
  Double_t       GetMichelEdep();  // edep in target by Michel e+

  void SetIncMuon4Pos(TLorentzVector xyzt); // set inital muon 4-pos
  void SetIncMuon3Pos(TVector3 xyz);        // set space component
  void SetIncMuonTime(double t);            // set time component
  void SetIncMuon4Mom(TLorentzVector pppe); // set inital muon 4-mom
  void SetIncMuon3Mom(TVector3 ppp);        // set space component
  void SetIncMuonEnergy(double e);          // set time component
  void SetMichelEdep(double e);             // edep by Michel e+

  public:
  ClassDef(TReadoutTarget,1)
};

#endif
