#ifndef READOUT_WORLD_HH
#define READOUT_WORLD_HH

#include "TVector3.h"
#include "TLorentzVector.h"
#include "TReadoutBranch.hh"

#include <iostream>
#include <string>

using namespace std;

class TReadoutWorld : public TReadoutBranch {

public:
  TReadoutWorld();
  //  ~TReadoutWorld();
  // the shared library won't build correctly if destructor is included!
private:
  UInt_t   fEventID;         // overall event number in the run
  UInt_t   fTotalPairsHit;   // total number of struck tile PAIRS
  TLorentzVector fMuonPrim4Pos; // initial position FOUR-vector of primary muon
  TLorentzVector fMuonPrim4Mom; // initial momentum FOUR-vector of primary muon
  TLorentzVector fMuonDecay4Pos; // position FOUR-vector of muon at decay
  UInt_t   fDecayVolID;      // integer identifying decay volume of muon  
public:
  UInt_t   GetEventID()         {return fEventID;}
  UInt_t   GetTotalPairsHit()   {return fTotalPairsHit;}
  UInt_t   GetDecayVolID()      {return fDecayVolID;}
  TLorentzVector GetMuonPrim4Pos(); // initial pos 4-vec of muon
  TVector3       GetMuonPrim3Pos(); // initial pos 3-vec of muon
  Double_t       GetMuonPrimTime(); // inital global time of muon
  TLorentzVector GetMuonPrim4Mom(); // initial mom 4-vec of muon
  TVector3       GetMuonPrim3Mom(); // initial mom 4-vec of muon
  Double_t       GetMuonPrimEnergy(); // initial [kin] energy of muon
  TLorentzVector GetMuonDecay4Pos();  // decay pos 4-vec of muon
  TVector3       GetMuonDecay3Pos();  // decay pos 3-vec of muon
  Double_t       GetMuonDecayTime();  // global time at muon decay

  void SetEventID(unsigned int id){fEventID = id;} // assign event ID
  void SetTotalPairsHit(unsigned int p){fTotalPairsHit = p;} // total hit pairs
  void SetDecayVolID(unsigned int id){fDecayVolID = id;} // record decay volume
  void SetDecayVolID(string volumeName); // figure out decay volume
  //  void SetDecayVolID(const char* volumeName); // figure out decay volume
  void SetMuonPrim4Pos(TLorentzVector xyzt); // set inital muon 4-pos
  void SetMuonPrim3Pos(TVector3 xyz);        // set space component
  void SetMuonPrimTime(double t);            // set time component
  void SetMuonPrim4Mom(TLorentzVector pppe); // set inital muon 4-mom
  void SetMuonPrim3Mom(TVector3 ppp);        // set space component
  void SetMuonPrimEnergy(double e);          // set time component
  void SetMuonDecay4Pos(TLorentzVector XYZT);// set decay 4-pos
  void SetMuonDecay3Pos(TVector3 XYZ);       // set space component
  void SetMuonDecayTime(double t);           // set time component

  bool ResetBranch(bool t);   // reset all member data to "null" values
  void CheckValues();
public:
  ClassDef(TReadoutWorld,1)
};

#endif
