// This code is to simulate the mulan experiment

#include "MulanDetectorConstruction.hh"
#include "MulanPhysicsList.hh"
#include "MulanPrimaryGeneratorAction.hh"
#include "MulanRunAction.hh"
#include "MulanEventAction.hh"
#include "MulanSteppingAction.hh"
#include "MulanSteppingVerbose.hh"

#include "G4RunManager.hh"
#include "G4UImanager.hh"
#include "G4UIterminal.hh"
#include "G4UItcsh.hh"
#include "G4UIGAG.hh"

#ifdef G4VIS_USE
#include "MulanVisManager.hh"
#endif

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

int main(int argc,char** argv) {

  //my Verbose output class
  G4VSteppingVerbose::SetInstance(new MulanSteppingVerbose);
  
  // Run manager
  G4cout<<"G4RunManager * runManager instantiating..."<<G4endl;
  G4RunManager * runManager = new G4RunManager;

  // UserInitialization classes (mandatory)
  MulanDetectorConstruction* Mulandetector = new MulanDetectorConstruction;
  runManager->SetUserInitialization(Mulandetector);
  runManager->SetUserInitialization(new MulanPhysicsList(Mulandetector));
  
#ifdef G4VIS_USE
  // Visualization, if you choose to have it!
  G4VisManager* visManager = new MulanVisManager;
  visManager->Initialize();
#endif
   
  // UserAction classes
  if(argc==3) //batch mode with file defined muons;
    {
      char* fileName = argv[2];
      runManager->SetUserAction(new 
		 MulanPrimaryGeneratorAction(Mulandetector, fileName));
    }
  else {
      runManager->SetUserAction(new 
		      MulanPrimaryGeneratorAction(Mulandetector,NULL));
    }

  runManager->SetUserAction(new MulanRunAction);  
  runManager->SetUserAction(new MulanEventAction);
  runManager->SetUserAction(new MulanSteppingAction);

  //Initialize G4 kernel
  G4cout <<"runManager initializing"<<G4endl;
  runManager->Initialize();
      
  //get the pointer to the User Interface manager 
  G4UImanager * UI = G4UImanager::GetUIpointer();  

  if(argc==1)
    // Define (G)UI terminal for interactive mode  
    { 
      // G4UIterminal is a (dumb) terminal.
      G4UIsession * session = 0;
#ifdef G4UI_USE_TCSH
//      session = new G4UIterminal(new G4UItcsh);      
      session = new G4UIGAG();      
#else
//      session = new G4UIterminal();
      session = new G4UIGAG();      
#endif    

      UI->ApplyCommand("/control/execute initInter.mac");    
      // this is the default macro that is run if none other specified

      session->SessionStart();
      delete session;
    }

  if(argc==3)
  {
    //    UI->ApplyCommand("/run/verbose 0");
    //    UI->ApplyCommand("/event/verbose 0");
    //    UI->ApplyCommand("/tracking/verbose 0");
    //    UI->ApplyCommand("/gun/particle mu+");
    //    char* doit;
    //    sprintf(doit,"/run/beamOn %s",argv[1]);
    //    UI->ApplyCommand(doit);
    
    G4String command = "/control/execute ";
    G4String fileName = argv[1];
    UI->ApplyCommand(command+fileName);
  }

  else
    // Batch mode
    { 
      G4String command = "/control/execute ";
      G4String fileName = argv[1];
      UI->ApplyCommand(command+fileName);
    }

#ifdef G4VIS_USE
  delete visManager;
#endif
  delete runManager;

  return 0;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

