// This code implementation is the intellectual property of
// the GEANT4 collaboration.
//
// By copying, distributing or modifying the Program (or any work
// based on the Program) you indicate your acceptance of this statement,
// and all its terms.
//
// $Id: MulanDetectorMessenger.cc,v 1.3 2004/02/03 21:05:58 asharp Exp $
// GEANT4 tag $Name:  $
// 
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

#include "MulanDetectorMessenger.hh"

#include "MulanDetectorConstruction.hh"
#include "G4UIdirectory.hh"
#include "G4UIcmdWithAString.hh"
#include "G4UIcmdWithADoubleAndUnit.hh"
#include "G4UIcmdWithoutParameter.hh"
#include "globals.hh"

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

MulanDetectorMessenger::MulanDetectorMessenger(MulanDetectorConstruction * myDet)
:myDetector(myDet)
{ 
  G4cout<<"MulanDetectorMessenger instantiating... ";
  mydetDir = new G4UIdirectory("/mydet/");
  mydetDir->SetGuidance("Mulan detector control.");

  G4String matList;
  const G4MaterialTable* matTbl = G4Material::GetMaterialTable();
  for(size_t i=0;i<G4Material::GetNumberOfMaterials();i++)
  {
    matList += (*matTbl)[i]->GetName();
    matList += " ";
  }
  
  TargMatCmd = new G4UIcmdWithAString("/mydet/setTargetMaterial",this);
  TargMatCmd->SetGuidance("Select Material of the Target.");
  TargMatCmd->SetParameterName("choice",false);
  TargMatCmd->SetCandidates(matList);
  TargMatCmd->AvailableForStates(PreInit,Idle);
    
  TargetMatEchoCmd = new G4UIcmdWithoutParameter("/mydet/echoTargetMaterial",this);
  TargetMatEchoCmd->SetGuidance("Prints the current Target volume material.");
  TargetMatEchoCmd->AvailableForStates(PreInit,Idle);

  FieldCmd = new G4UIcmdWithADoubleAndUnit("/mydet/setMagneticField",this);  
  FieldCmd->SetGuidance("Define magnetic field.");
  FieldCmd->SetGuidance("Magnetic field will be in Z direction.");
  FieldCmd->SetParameterName("Bz",false);
  FieldCmd->SetDefaultUnit("tesla");
  FieldCmd->SetUnitCategory("Magnetic flux density");
  FieldCmd->AvailableForStates(PreInit,Idle);  

  FieldEchoCmd = new G4UIcmdWithoutParameter("/mydet/echoMagneticField",this);
  FieldEchoCmd->SetGuidance("Prints current target magnetic field setting.");
  FieldEchoCmd->AvailableForStates(PreInit,Idle);

  WorldMatCmd = new G4UIcmdWithAString("/mydet/setWorldMaterial",this);
  WorldMatCmd->SetGuidance("Select Material for the world volume.");
  WorldMatCmd->SetParameterName("choice",false); // parameter not omittable
  WorldMatCmd->SetCandidates(matList);
  WorldMatCmd->AvailableForStates(PreInit,Idle);

  WorldMatEchoCmd = new G4UIcmdWithoutParameter("/mydet/echoWorldMaterial",this);
  WorldMatEchoCmd->SetGuidance("Prints the current World volume material.");
  WorldMatEchoCmd->AvailableForStates(PreInit,Idle);

  UpdateCmd = new G4UIcmdWithoutParameter("/mydet/update",this);
  UpdateCmd->SetGuidance("Update detector geometry parameters.");
  UpdateCmd->SetGuidance("This command MUST be applied before \"beamOn\" ");
  UpdateCmd->SetGuidance("if you changed any value(s).");
  UpdateCmd->AvailableForStates(Idle);

  G4cout<<"done"<<G4endl;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

MulanDetectorMessenger::~MulanDetectorMessenger()
{
  delete TargMatCmd;
  delete TargetMatEchoCmd;
  delete FieldCmd;
  delete WorldMatCmd;
  delete WorldMatEchoCmd;
  delete UpdateCmd;
  delete mydetDir;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

void MulanDetectorMessenger::SetNewValue(G4UIcommand * command,G4String newValues)
{ 
  if( command == TargMatCmd )
    { myDetector->SetTargetMaterial(newValues);}
     
  else if( command == TargetMatEchoCmd )
    { myDetector->EchoTargetMaterial();}

  else if( command == FieldCmd )
    { myDetector->SetMagField(FieldCmd->GetNewDoubleValue(newValues));}

  else if( command == FieldEchoCmd )
    { myDetector->EchoMagField();}

  else if( command == WorldMatCmd )
    { myDetector->SetWorldMaterial(newValues);}

  else if( command == WorldMatEchoCmd )
    { myDetector->EchoWorldMaterial();}

  else if( command == UpdateCmd )
    { myDetector->UpdateGeometry(); }


  else G4cout <<"That command not found."<<G4endl;

}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....
