// $Id: MulanDetectorSD.cc,v 1.37 2004/03/25 16:30:11 asharp Exp $
// GEANT4 tag $Name:  $
//
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

#include "MulanDetectorSD.hh"
#include "MulanDetectorSD.ecc"
#include "MulanRunAction.hh"
#include "MulanPrimaryGeneratorAction.hh"
#include "MulanEventAction.hh"
//#include "MulanTrackInformation.hh"
#include "G4HCofThisEvent.hh"
#include "G4THitsCollection.hh"
#include "G4ParticleDefinition.hh"
#include "G4Step.hh"
#include "G4ThreeVector.hh"
#include "G4SDManager.hh"
#include "G4ios.hh"
#include "G4VProcess.hh"
#include "stdio.h"
#include "math.h"
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

MulanDetectorSD::MulanDetectorSD(G4String name)
:G4VSensitiveDetector(name)
{
  G4cout<<"MulanDetectorSD instantiating...";
  G4String HCname = name;
  collectionName.insert(HCname="detectorCollection");

  /////////////////////////////////////////
  // Declare and initialize ROOT objects //
  /////////////////////////////////////////
  rootFile = new TFile("/.autofs/home/asharp/work/mulan/G4file.root", 
		     "RECREATE");
  //  rootFile = new TFile("/.autofs/home/rmcnabb/mulan/G4/mulan/G4file.root", 
  //		     "RECREATE");
  TotalEdepPerEvent = new TH1D("TotalEdepPerEvent", "Edep in Detector",
			       200, 0., 5.0);
  TotalEdepPerEvent->GetXaxis()->SetTitle("[MeV]");
  NumTilesHit = new TH1D("NumTilesHit", "Number Tile Strikes per Event",
		       21, -0.5, 20.5);
  NumTilesHitEdepCut = new TH1D("NumTilesHitEdepCut", "Number Tile Strikes per Event",
		       21, -0.5, 20.5);
  EdepPerTilePair = new TH2D("EdepPerTilePair", "Edep in Each Tile", 
			     340, 0.5, 340.5, 200, 0, 5);
  EdepPerTilePair->GetXaxis()->SetTitle("Tile Serial Number");
  EdepPerTilePair->GetYaxis()->SetTitle("Edep [MeV]");

  DataTree = new TTree("DataTree","ROOT tree for G4 data");
  //  DataTree->SetDirectory(rootFile);
  worldBranch = new TReadoutWorld();
  targetBranch = new TReadoutTarget();
  tileBranch1 = new TReadoutTilePair();
  tileBranch2 = new TReadoutTilePair();
  tileBranch3 = new TReadoutTilePair();
  tileBranch4 = new TReadoutTilePair();
  DataTree->Branch("worldBranch","TReadoutWorld",&worldBranch);
  DataTree->Branch("targetBranch","TReadoutTarget",&targetBranch);
  DataTree->Branch("tileBranch1","TReadoutTilePair",&tileBranch1);
  DataTree->Branch("tileBranch2","TReadoutTilePair",&tileBranch2);
  DataTree->Branch("tileBranch3","TReadoutTilePair",&tileBranch3);
  DataTree->Branch("tileBranch4","TReadoutTilePair",&tileBranch4);
  DataTree->Print("tileBranch1*");  // goes to stdout to see tree structure
  
  G4cout<<" done"<<G4endl;
  evtIndex = -1;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

MulanDetectorSD::~MulanDetectorSD()
{ 
  G4cout<<"This is MulanDetectorSD destructor\n";
  rootFile->Write();
  rootFile->Close();
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

void MulanDetectorSD::Initialize(G4HCofThisEvent* HCE)
{
  detectorCollection = new MulanDetectorHitsCollection(SensitiveDetectorName,collectionName[0]); 
  static G4int HCID = -1;
  if(HCID<0)
  { 
    HCID = G4SDManager::GetSDMpointer()->GetCollectionID(collectionName[0]);
  }
  HCE->AddHitsCollection( HCID, detectorCollection ); 
  // resetting variables to be used in each step
  badFirstStep = 0;
  T_ID = -1;
  evtIndex++; // events counter starts at 0.
  Prim4VecSet = false;
  Decay4VecSet = false;
  DecayVolSet = false;
}
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

G4bool MulanDetectorSD::ProcessHits(G4Step* aStep,
				    G4TouchableHistory* ROhist)
{
  /////////////////////////////////////////////////////////////////////
  // This function is called when a particle takes a step in a       //
  // sensitive volume. It looks up relevant information about the    //
  // particle, its location, what it's doing, etc. and stores it in a//
  // MulanDetectorHit object (a new instance of it called newHit is  //
  // made each time). This information is recovered later when       //
  // EndOfEvent() is called. If you want to look at the info while   // 
  // it's stored in the hit collection (a vector of MulanDetectorHit //
  // objects compiled for each event), use the methods of the class  //
  // MulanDetectorHit, like Print().                                 //
  /////////////////////////////////////////////////////////////////////

  if(badFirstStep == 1) return false;
  particleName = aStep->GetTrack()->GetDefinition()->GetParticleName();

  // if the particle at hand is a neutrino, just kill it and move on
  if(!strcmp(particleName,"anti_nu_mu")||!strcmp(particleName,"nu_mu")||
     !strcmp(particleName,"anti_nu_e")||!strcmp(particleName,"nu_e")) { 
    //      G4cout <<"killing nu track and returning true"<<G4endl;
    aStep->GetTrack()->SetTrackStatus(
	      (const G4TrackStatus)fKillTrackAndSecondaries);
    return true; 
  }

  parentID = aStep->GetTrack()->GetParentID();
  currentVolume = aStep->GetPreStepPoint()->GetPhysicalVolume();
  volumeName = currentVolume->GetName();
  const G4VProcess* stepProcess=aStep->GetPostStepPoint()->GetProcessDefinedStep();
  G4String stepProcessName = "None";
  if(stepProcess != NULL) {stepProcessName = stepProcess->GetProcessName();}
  const G4VProcess* creatorProcess = aStep->GetTrack()->GetCreatorProcess();
  G4String creatorProcessName = "None";
  if(creatorProcess != NULL) {
    creatorProcessName = creatorProcess->GetProcessName();
  }
  MulanDetectorHit* newHit = new MulanDetectorHit();
  // these numbers we know right away
  newHit->SetTrackNumber(aStep->GetTrack()->GetTrackID());
  newHit->SetEdep(aStep->GetTotalEnergyDeposit());
  newHit->SetParticleMom(aStep->GetPreStepPoint()->GetMomentum());
  newHit->SetHitPosition(aStep->GetPreStepPoint()->GetPosition());
  newHit->SetHitVol(volumeName);
  newHit->SetParticleEnergy(aStep->GetTrack()->GetKineticEnergy());
  newHit->SetVertexPos(aStep->GetTrack()->GetVertexPosition());

  // set the names of process used in this step, and 
  // process that created the current particle
  newHit->SetStepProcName(stepProcessName);
  if(creatorProcess!=NULL) {
    G4String *creator = &creatorProcessName;
    newHit->SetCreatorProcName(*creator);
  }
  else {newHit->SetCreatorProcName(creatorProcessName);}


  // global time is overall world time since generation of primary
  // muon. local time is length of time this particular particle track
  // has been alive. therefore the global time when this track was
  // created is equal to the current global time minus the current
  // local time.
  thisGlobalTime = aStep->GetPreStepPoint()->GetGlobalTime();
  thisLocalTime = aStep->GetPreStepPoint()->GetLocalTime();
  newHit->SetGlobalTime(thisGlobalTime);
  newHit->SetLocalTime(thisLocalTime);
  newHit->SetVertexTime(abs(thisGlobalTime - thisLocalTime)/nanosecond);

  newHit->SetVertexMom(aStep->GetTrack()->GetVertexMomentumDirection());
  newHit->SetVertexEnergy(aStep->GetTrack()->GetVertexKineticEnergy());

  // calculate the serial number here
  G4int SerialNumber = 0; //hack
  if((!strcmp(volumeName,"InnerTileElement")  
	   || !strcmp(volumeName,"OuterTileElement")
	   || !strcmp(volumeName,"InnerSPTileElement")
	   || !strcmp(volumeName,"OuterSPTileElement"))) {
  if(aStep->GetPreStepPoint()->GetPhysicalVolume()->GetMother() == NULL)
    { G4cerr <<"Mother volume was NULL"<<G4endl; }
  else hitS_ID =
	 aStep->GetPreStepPoint()->GetTouchable()->GetReplicaNumber(1);
  hitT_ID = aStep->GetPreStepPoint()->GetTouchable()->GetReplicaNumber(0);
  const G4int Zipcode = GetZipcode(volumeName,hitS_ID,hitT_ID);
  SerialNumber = GetSerialNumber(Zipcode);
  }
  newHit->SetSerialNumber(SerialNumber);

  // calculate the particle type number
  G4int type = GetParticleType(particleName,parentID);
  newHit->SetParticleType(type);

  // these statements are for implementing another class. They cause a
  // seg fault at this time, so don't try to use them.
//    MulanTrackInformation* info = new MulanTrackInformation();
//    info->Print();
//    info = (MulanTrackInformation*)(aStep->GetTrack()->GetUserInformation());
//    info->GetOriginalTrackID();
//    info->Print();
//    G4cout << " Original Track ID " << info->GetOriginalTrackID() << G4endl;


  detectorCollection->insert(newHit);

  return true;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

void MulanDetectorSD::EndOfEvent(G4HCofThisEvent* HCE)
{
  ////////////////////////////////////////////////////////////////////
  // This function is called when each Event has completed (i.e., a //
  // primary muon is generated and tracked, then all of its daughter//
  // and granddaughter particles are tracked). It sets the event    //
  // number index in the tree, then calls ProcessEventHC() to fill  //
  // other leaves and branches in the tree.                         //
  ////////////////////////////////////////////////////////////////////

  // evtIndex is incremented in Initialize() fcn. Starts with 0, not 1.
  G4cout<<"Doing ProcessEventHC on event "<<evtIndex<<G4endl;
  worldBranch->SetEventID(evtIndex);

  ProcessEventHC(HCE);
  //    G4int NbHits = detectorCollection->entries(); 
  //    G4cout << "\n-------->Hits Collection: in this event"
  //                 << there are " << NbHits << " hits in the detectors: "
  //                 << G4endl;
  //    for (G4int i=0;i<NbHits;i++) {
  //	(*detectorCollection)[i]->Print();
  //    }
}

void MulanDetectorSD::ProcessEventHC(G4HCofThisEvent* hitCollections)
{
  //////////////////////////////////////////////////////////////////////
  // Called from EndOfEvent(). Does the processing on each of the hit //
  // collections for each event. There is 1 hit collection for each   //
  // sensitive detector declared in MulanDetectorConstruction. Each   //
  // hit collection is a vector of MulanDetectorHit objects containing//
  // relevant information about the steps of each particle within a   //
  // sensitive volume. Retrieves information from the MulanDetectorHit//
  // objects and stores into branches/leaves of DataTree.             //
  //////////////////////////////////////////////////////////////////////

  const G4int ARRAY_LENGTH = 50;
  totalEdep = 0;
  singleEdep = 0;
  tilesHit = 0;
  tilesHitECut = 0;
  serialNum = 0;
//    G4int partnerSerialNum;
  hitTime = -1.5;
  G4int serialList[ARRAY_LENGTH] = {0};
  G4int numTimesHitList[ARRAY_LENGTH] = {0};
  G4double totalEdepList[ARRAY_LENGTH] = {0};
  G4double hitTimeList[ARRAY_LENGTH] = {0};
  counter = 0;
  for(G4int k=0; k<ARRAY_LENGTH; k++){serialList[k] = -1;}
  MulanDetectorHitsCollection *aHitCollWithinThisEvent;

  // loop over the Hit Collections (1 for each SD in DetConstruction)
  G4int NCollections = hitCollections->GetNumberOfCollections();
  for(G4int n = 0; n < NCollections; n++) {
    aHitCollWithinThisEvent = (MulanDetectorHitsCollection*) 
      hitCollections->GetHC(n);
    G4int EntriesWithinHitColl = aHitCollWithinThisEvent->entries();
    
    // loop over the hits within the hit collections
    for(G4int ent = 0; ent < EntriesWithinHitColl; ent++) {
      G4cout<<"Starting entry "<<ent<<" of "<<EntriesWithinHitColl<<G4endl;
      serialNum = (*aHitCollWithinThisEvent)[ent]->GetSerialNumber();
      G4int thisType = (*aHitCollWithinThisEvent)[ent]->GetParticleType();
      G4String thisProcess = (*aHitCollWithinThisEvent)[ent]->GetStepProcName();
      G4String thisVolume = (*aHitCollWithinThisEvent)[ent]->GetHitVolName();

      G4double localT = (*aHitCollWithinThisEvent)[ent]->GetLocalTime();
      G4double globalT = (*aHitCollWithinThisEvent)[ent]->GetGlobalTime();
      G4double vertexT = //equals globalT-localT
	(*aHitCollWithinThisEvent)[ent]->GetVertexTime();

      if(thisType == 4 && !strcmp(thisProcess,"Decay") && !DecayVolSet) {
	// if the primary muon decayed in this step, get the ID of volume
	worldBranch->SetDecayVolID(thisVolume);
	DecayVolSet = true;
      }
      if(thisType == 4 && !Prim4VecSet) {
	// this is primary muon; we want to find
	// its vertex position and momentum.
	G4ThreeVector vtxPos = (*aHitCollWithinThisEvent)[ent]->GetVertexPos();
	worldBranch->SetMuonPrim3Pos(TVector3(vtxPos.x(),vtxPos.y(),vtxPos.z()));
  	if(vertexT<1e-10) vertexT = 0.0; // truncate very small numbers
	worldBranch->SetMuonPrimTime(vertexT);

	G4ThreeVector vtxMom = (*aHitCollWithinThisEvent)[ent]->GetVertexMom();
	worldBranch->SetMuonPrim3Mom(TVector3(vtxMom.x(),vtxMom.y(),vtxMom.z()));
	G4double vtxE = (*aHitCollWithinThisEvent)[ent]->GetVertexEnergy();
	if(vtxE<1e-10) vtxE = 0.0; // truncate very small numbers
	worldBranch->SetMuonPrimEnergy(vtxE);
	Prim4VecSet = true;  // flag to say this info recorded once
	if(Prim4VecSet)	G4cout<<"Prim4VecSet is now true"<<G4endl;
	else G4cout<<"Prim4VecSet is now false"<<G4endl;
      }

      else if(thisType == 1 && !Decay4VecSet) {
	// this is a Michel; find its creation vertex
	// for the muon decay position information
	G4ThreeVector decayPos = (*aHitCollWithinThisEvent)[ent]->GetVertexPos();
	G4double decayT = (*aHitCollWithinThisEvent)[ent]->GetVertexTime();
	if(decayT<1e-10) decayT = 0.0; // truncate very small numbers
	worldBranch->SetMuonDecayTime(decayT);
	worldBranch->SetMuonDecay3Pos(TVector3(decayPos.x(),decayPos.y(),decayPos.z()));
	Decay4VecSet = true;  // flag to say this info recorded once
	if(Decay4VecSet)	G4cout<<"Decay4VecSet is now true"<<G4endl;
	else G4cout<<"Decay4VecSet is now false"<<G4endl;
      }

      if(serialNum > 0) {
	G4cout <<"hit something: Serial number is "<<serialNum<<G4endl;
	singleEdep = (*aHitCollWithinThisEvent)[ent]->GetEdep();
	if(singleEdep>0.125) { tilesHitECut++; }
	
	counter = 0;
	// search the struck tiles array for same serialNum
	while(serialList[counter]!= serialNum 
	      && serialList[counter] != -1
	      && counter<ARRAY_LENGTH) {counter++;}
	serialList[counter] = serialNum;
	numTimesHitList[counter] += 1;
	totalEdepList[counter] += singleEdep;
	hitTimeList[counter] = globalT;
	G4cout<<"serialList = ";
	for(G4int i=0; i<counter+1; i++) { 
	  G4cout<<"{"<<serialList[i]<<",";
	  G4cout<<numTimesHitList[i]<<",";
	  G4cout<<totalEdepList[i]<<",";
	  G4cout<<hitTimeList[i]<<"}";
	}
      } // end of if(serialNum > 0) statement

      else {	// do something with the non-tile events
	G4cout <<"did not hit a tile\n";

      }
      
    } // end of loop over Entries in the hit collection
  } // end of loop over the "multiple" hit collections
  
  //   putting these histogram fills outside the hit 
  //   collections loop is a safety; however not necessary
  //   since we only have one sensitive detector instantiated.
  
  // fill the histograms with data from each hit
  for(G4int c=0; c<ARRAY_LENGTH; c++) {
    totalEdep += totalEdepList[c];
//      tilesHit  += numTimesHitList[c];
//      EdepPerTilePair->Fill(serialList[c], numTimesHitList[c]);
  }
  G4int c2=0;
  while(c2<ARRAY_LENGTH && serialList[c2]!= -1) {
    serialNum = serialList[c2];
    singleEdep = totalEdepList[c2];
    c2++;
  }
  worldBranch->SetTotalPairsHit((unsigned)c2);

  TotalEdepPerEvent->Fill(totalEdep);
  TotalEdepPerEvent->Write(TotalEdepPerEvent->GetName(),TObject::kOverwrite);
  NumTilesHit->Fill(tilesHit);
  NumTilesHit->Write(NumTilesHit->GetName(),TObject::kOverwrite);
  NumTilesHitEdepCut->Fill(tilesHitECut);
  NumTilesHitEdepCut->Write(NumTilesHitEdepCut->GetName(),TObject::kOverwrite);
  EdepPerTilePair->Write(EdepPerTilePair->GetName(),TObject::kOverwrite);

  G4cout<<"Prim4VecSet = "<<Prim4VecSet
	<<", Decay4VecSet = "<<Decay4VecSet
	<<", DecayVolSet = "<<DecayVolSet
	<<G4endl<<G4endl<<G4endl;
  worldBranch->CheckValues();

  DataTree->Fill();
  DataTree->Write(DataTree->GetName(),TObject::kOverwrite);

  worldBranch->ResetBranch(true);
}



//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....
G4int MulanDetectorSD::GetParticleType(G4String particleName,
				      G4String creatorProcessName)
{
  if(!strcmp(particleName,"e+") && !strcmp(creatorProcessName,"Decay"))
    {return 1;}
  else if(!strcmp(particleName,"e-"))    {return 2;}
  else if(!strcmp(particleName,"gamma")) {return 3;}
  else if(!strcmp(particleName,"mu+"))   {return 4;}
  else if(!strcmp(particleName,"e+") && strcmp(creatorProcessName,"Decay"))
    {return 5;}
  else {return 6;}
}
G4int MulanDetectorSD::GetParticleType(G4String particleName,
				       G4int parentID)
{
  if(!strcmp(particleName,"e+") && parentID == 1)
    {return 1;}
  else if(!strcmp(particleName,"e-"))    {return 2;}
  else if(!strcmp(particleName,"gamma")) {return 3;}
  else if(!strcmp(particleName,"mu+") && parentID == 0)   {return 4;}
  else if(!strcmp(particleName,"e+") && parentID > 1)
    {return 5;}
  else {return 6;}
}








