#include "MulanMaterialStepCut.hh"

#include "G4Step.hh"
#include "G4UserLimits.hh"
#include "G4VParticleChange.hh"
#include "G4EnergyLossTables.hh"
#include "G4Track.hh"

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....


//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

MulanMaterialStepCut::MulanMaterialStepCut(const G4String& aName): 
    	G4VDiscreteProcess(aName),fMaxStep(DBL_MAX),fAirStep(DBL_MAX),
        fHeStep(DBL_MAX),fSilverStep(DBL_MAX),fScintStep(DBL_MAX),
	fChamberGasStep(DBL_MAX),fVacuumStep(DBL_MAX),fMylarStep(DBL_MAX),
        fSulfurStep(DBL_MAX),fAlStep(DBL_MAX)
{
  //  G4cout<<"MulanMaterialStepCut::MulanMaterialStepCut begun."<<G4endl;
   if (verboseLevel) {
     G4cout << GetProcessName() << " is created "<< G4endl;
   }
   //  G4cout<<"MulanMaterialStepCut::MulanMaterialStepCut ended."<<G4endl;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

MulanMaterialStepCut::~MulanMaterialStepCut()
{}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

MulanMaterialStepCut::MulanMaterialStepCut(MulanMaterialStepCut& right)
    :G4VDiscreteProcess(right)
{}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

G4double MulanMaterialStepCut::PostStepGetPhysicalInteractionLength(
                             const G4Track& aTrack,
                             G4double   previousStepSize,
                             G4ForceCondition* condition
                            )
{
  // condition is set to "Not Forced"
  *condition = NotForced;

   G4double ProposedStep = fMaxStep;
   G4Material* Mater = aTrack.GetNextVolume()->GetLogicalVolume()->GetMaterial();
   G4String name = Mater->GetName();
   if ((name == "Air") && (fAirStep < DBL_MAX)) ProposedStep = fAirStep;
   if ((name == "He") && (fHeStep < DBL_MAX)) ProposedStep = fHeStep;
   if ((name == "Silver") && (fSilverStep < DBL_MAX)) ProposedStep = fSilverStep;
   if ((name == "WireChamberGas") && (fChamberGasStep < DBL_MAX)) ProposedStep = fChamberGasStep;
   if ((name == "Vacuum") && (fVacuumStep < DBL_MAX)) ProposedStep = fVacuumStep;
   if ((name == "Mylar") && (fMylarStep < DBL_MAX)) ProposedStep = fMylarStep;
   if ((name == "Sulfur") && (fSulfurStep <DBL_MAX)) ProposedStep = fSulfurStep;
   if ((name == "BC-404") && (fScintStep < DBL_MAX))  ProposedStep = fScintStep;
   if ((name == "BC-408") && (fScintStep < DBL_MAX))  ProposedStep = fScintStep;
   if ((name == "Al") && (fAlStep < DBL_MAX))  ProposedStep = fScintStep;

   return ProposedStep;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

G4VParticleChange* MulanMaterialStepCut::PostStepDoIt(
                             const G4Track& aTrack,
                             const G4Step&
                            )
{
   // do nothing
   aParticleChange.Initialize(aTrack);
   return &aParticleChange;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

inline G4double MulanMaterialStepCut::GetMeanFreePath(const G4Track& aTrack,
                             G4double   previousStepSize,
                             G4ForceCondition* condition
                            )
{
  return 0.;
}

