// This code implementation is the intellectual property of
// the GEANT4 collaboration.
//
// By copying, distributing or modifying the Program (or any work
// based on the Program) you indicate your acceptance of this statement,
// and all its terms.
//
// $Id: MulanPhysicsList.cc,v 1.9 2004/02/27 23:58:24 asharp Exp $
// GEANT4 tag $Name:  $
//
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

#include "globals.hh"
#include "MulanPhysicsList.hh"

#include "G4ParticleDefinition.hh"
#include "G4ParticleWithCuts.hh"
#include "G4ProcessManager.hh"
#include "G4ProcessVector.hh"
#include "G4ParticleTypes.hh"
#include "G4ParticleTable.hh"
#include "G4Material.hh"
#include "G4ios.hh"
#include "g4std/iomanip"                

#ifndef ALL_PHYSICS_ON    // this is a control switch for changing
#define ALL_PHYSICS_ON 0  // to the mode of all physics processes
#endif                    // turned off. there are 2 definitions of ConstructEM()
// to turn off the physics processes, change the definition of 1 to 0
// but do NOT change anything else!

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

MulanPhysicsList::MulanPhysicsList(MulanDetectorConstruction* detector)
		 :G4VUserPhysicsList()
{
  fdetector = detector;
  defaultCutValue = 1.0*cm;
   SetVerboseLevel(0);
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

MulanPhysicsList::~MulanPhysicsList()
{}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

void MulanPhysicsList::ConstructParticle()
{
  // In this method, static member functions should be called
  // for all particles which you want to use.
  // This ensures that objects of these particle types will be
  // created in the program. 

  ConstructBosons();
  ConstructLeptons();
  ConstructBaryons();

}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

void MulanPhysicsList::ConstructBosons()
{
  // gamma
  G4Gamma::GammaDefinition();
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

void MulanPhysicsList::ConstructBaryons()
{
  G4AntiProton::AntiProtonDefinition();
  G4Proton::ProtonDefinition();
}

void MulanPhysicsList::ConstructLeptons()
{
  // leptons
  //  e+/-
  G4Electron::ElectronDefinition();
  G4Positron::PositronDefinition();
  // mu+/-
  G4MuonPlus::MuonPlusDefinition();
  G4MuonMinus::MuonMinusDefinition();
  // nu_e
  G4NeutrinoE::NeutrinoEDefinition();
  G4AntiNeutrinoE::AntiNeutrinoEDefinition();
  // nu_mu
  G4NeutrinoMu::NeutrinoMuDefinition();
  G4AntiNeutrinoMu::AntiNeutrinoMuDefinition();
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....


void MulanPhysicsList::ConstructProcess()
{
  AddTransportation();
  ConstructEM();
  ConstructGeneral();
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

#include "G4ComptonScattering.hh"
#include "G4GammaConversion.hh"
#include "G4PhotoElectricEffect.hh"
#include "G4MultipleScattering.hh"
#include "G4hIonisation.hh"
#include "G4eIonisation.hh"
#include "G4eBremsstrahlung.hh"
#include "G4eplusAnnihilation.hh"

#include "G4MuIonisation.hh"
#include "G4MuBremsstrahlung.hh"
#include "G4MuPairProduction.hh"
#include "G4UserSpecialCuts.hh"

#include "G4Decay.hh"
#include "MulanMaterialStepCut.hh"

#include "G4LowEnergyIonisation.hh"
#include "G4LowEnergyBremsstrahlung.hh"
#include "G4LowEnergyCompton.hh"
#include "G4LowEnergyPhotoElectric.hh"
#include "G4LowEnergyGammaConversion.hh"

#include "G4Scintillation.hh"

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

#if ALL_PHYSICS_ON  
// first version of the ConstructEM() function.
// if you want to turn off the physics processes, change the 
// definition of ALL_PHYSICS_ON to 0. DO NOT change anything here!
void MulanPhysicsList::ConstructEM()
{
  //  G4cout<<"Entered MulanPhysicsList::ConstructEM()"<<G4endl;
  theParticleIterator->reset();
  while( (*theParticleIterator)() ){
    G4ParticleDefinition* particle = theParticleIterator->value();
    G4String particleName = particle->GetParticleName();
    G4ProcessManager* pmanager = particle->GetProcessManager();
 
    if (particleName == "gamma") {
    // gamma
      pmanager->AddDiscreteProcess(new G4GammaConversion());
      pmanager->AddDiscreteProcess(new G4ComptonScattering());      
      pmanager->AddDiscreteProcess(new G4PhotoElectricEffect());
//      pmanager->AddDiscreteProcess(new G4LowEnergyGammaConversion());
//      pmanager->AddDiscreteProcess(new G4LowEnergyCompton());      
//      pmanager->AddDiscreteProcess(new G4LowEnergyPhotoElectric());

    } else if (particleName == "e-") {
    //electron
      G4VProcess* theeminusMultipleScattering = new G4MultipleScattering();
      G4VProcess* theeminusIonisation         = new G4eIonisation();
      G4VProcess* theeminusBremsstrahlung     = new G4eBremsstrahlung();
//      G4VProcess* theeminusIonisation         = new G4LowEnergyIonisation();
//      G4VProcess* theeminusBremsstrahlung     = new G4LowEnergyBremsstrahlung();

      //
      // add processes
      pmanager->AddProcess(theeminusMultipleScattering);
      pmanager->AddProcess(theeminusIonisation);
      pmanager->AddProcess(theeminusBremsstrahlung);
      //      
      // set ordering for AlongStepDoIt
      pmanager->SetProcessOrdering(theeminusMultipleScattering, idxAlongStep,1);
      pmanager->SetProcessOrdering(theeminusIonisation,         idxAlongStep,2);
      //
      // set ordering for PostStepDoIt
      pmanager->SetProcessOrdering(theeminusMultipleScattering, idxPostStep,1);
      pmanager->SetProcessOrdering(theeminusIonisation,         idxPostStep,2);
      pmanager->SetProcessOrdering(theeminusBremsstrahlung,     idxPostStep,3);

    } else if (particleName == "e+") {
    //positron
      G4VProcess* theeplusMultipleScattering = new G4MultipleScattering();
      G4VProcess* theeplusIonisation         = new G4eIonisation();
      G4VProcess* theeplusBremsstrahlung     = new G4eBremsstrahlung();
//      G4VProcess* theeplusIonisation         = new G4LowEnergyIonisation();
//      G4VProcess* theeplusBremsstrahlung     = new G4LowEnergyBremsstrahlung();
      G4VProcess* theeplusAnnihilation       = new G4eplusAnnihilation();
      //
      // add processes
      pmanager->AddProcess(theeplusMultipleScattering);
      pmanager->AddProcess(theeplusIonisation);
      pmanager->AddProcess(theeplusBremsstrahlung);
      pmanager->AddProcess(theeplusAnnihilation);
      //
      // set ordering for AtRestDoIt
      pmanager->SetProcessOrderingToFirst(theeplusAnnihilation,idxAtRest);
      //
      // set ordering for AlongStepDoIt
      pmanager->SetProcessOrdering(theeplusMultipleScattering,idxAlongStep,1);
      pmanager->SetProcessOrdering(theeplusIonisation,idxAlongStep,2);
      //
      // set ordering for PostStepDoIt
      pmanager->SetProcessOrdering(theeplusMultipleScattering,idxPostStep,1);
      pmanager->SetProcessOrdering(theeplusIonisation,idxPostStep,2);
      pmanager->SetProcessOrdering(theeplusBremsstrahlung,idxPostStep,3);
      pmanager->SetProcessOrdering(theeplusAnnihilation,idxPostStep,4);
      // make a user step length definition for positrons
      MulanMaterialStepCut* theEplusStepCut = new MulanMaterialStepCut();  
      //      theEplusStepCut->SetHeStep(10*cm);
      //      theEplusStepCut->SetAirStep(10*cm);
      //      theEplusStepCut->SetChamberGasStep(1*mm);
      //      theEplusStepCut->SetVacuumStep(10*cm);
      //      theEplusStepCut->SetMylarStep(50*micrometer);
      //      theEplusStepCut->SetScintStep(5*micrometer);
      //      theEplusStepCut->SetSulfurStep(.1*mm);
      //      theEplusStepCut->SetSilverStep(0.5*mm);
      //      theEplusStepCut->SetAlStep(0.5*mm);
      //      theEplusStepCut->SetMaxStep(.8*mm);
      pmanager->AddProcess(theEplusStepCut, -1, -1, 5);


    } else if( particleName == "mu+" || 
               particleName == "mu-"    ) {
    //muon

      G4VProcess* aMultipleScattering = new G4MultipleScattering();
      G4VProcess* aBremsstrahlung     = new G4MuBremsstrahlung();
      G4VProcess* aPairProduction     = new G4MuPairProduction();
      G4VProcess* anIonisation        = new G4MuIonisation();
      
      //
      // add processes
      pmanager->AddProcess(anIonisation);
      pmanager->AddProcess(aMultipleScattering);
      pmanager->AddProcess(aBremsstrahlung);
      pmanager->AddProcess(aPairProduction);
      //
      // set ordering for AlongStepDoIt
      pmanager->SetProcessOrdering(aMultipleScattering, idxAlongStep,1);
      pmanager->SetProcessOrdering(anIonisation,        idxAlongStep,2);
      //
      // set ordering for PostStepDoIt
      pmanager->SetProcessOrdering(aMultipleScattering, idxPostStep,1);
      pmanager->SetProcessOrdering(anIonisation,        idxPostStep,2);
      pmanager->SetProcessOrdering(aBremsstrahlung,     idxPostStep,3);
      pmanager->SetProcessOrdering(aPairProduction,     idxPostStep,4);

      MulanMaterialStepCut* theMuonStepCut = new MulanMaterialStepCut();
      theMuonStepCut->SetHeStep(10*cm);
      theMuonStepCut->SetAirStep(10*cm);
//      theMuonStepCut->SetChamberGasStep(1*mm);
//      theMuonStepCut->SetVacuumStep(10*cm);
      theMuonStepCut->SetMylarStep(50*micrometer);
      theMuonStepCut->SetScintStep(50*micrometer);
//      theMuonStepCut->SetSulfurStep(.1*mm);
      theMuonStepCut->SetSilverStep(.025*mm);
      theMuonStepCut->SetAlStep(5*nanometer);
//      theMuonStepCut->SetMaxStep(.8*mm);
      pmanager->AddProcess(theMuonStepCut, -1, -1, 5);
      //      G4cout<<"Set all the forced step lengths"<<G4endl;
    } else if (particleName == "proton") {
    //proton
      G4VProcess* protonMultipleScattering = new G4MultipleScattering();
      G4VProcess* protonIonisation         = new G4hIonisation();

      //
      // add processes
      pmanager->AddProcess(protonMultipleScattering);
      pmanager->AddProcess(protonIonisation);
      //
      // set ordering for AtRestDoIt
      //
      // set ordering for AlongStepDoIt
      pmanager->SetProcessOrdering(protonMultipleScattering,idxAlongStep,1);
      pmanager->SetProcessOrdering(protonIonisation,idxAlongStep,2);
      //
      // set ordering for PostStepDoIt
      pmanager->SetProcessOrdering(protonMultipleScattering,idxPostStep,1);
      pmanager->SetProcessOrdering(protonIonisation,idxPostStep,2);      
    }
  }
}

#else 
// meaning if the ALL_PHYSICS_ON flag is 0
// then use this version of the ConstructEM() function
void MulanPhysicsList::ConstructEM()
{
  theParticleIterator->reset();
  while( (*theParticleIterator)() ){
    G4ParticleDefinition* particle = theParticleIterator->value();
    G4String particleName = particle->GetParticleName();
    G4ProcessManager* pmanager = particle->GetProcessManager();
 
    if (particleName == "gamma") {
    // gamma
    }

    else if (particleName == "e-") {
    //electron
      G4VProcess* theeminusMultipleScattering = new G4MultipleScattering();
      G4VProcess* theeminusIonisation         = new G4eIonisation();
      G4VProcess* theeminusBremsstrahlung     = new G4eBremsstrahlung();
//      G4VProcess* theeminusIonisation         = new G4LowEnergyIonisation();
//      G4VProcess* theeminusBremsstrahlung     = new G4LowEnergyBremsstrahlung();

      //
      // add processes
      // (removed)
    }
    else if (particleName == "e+") {
    //positron
      G4VProcess* theeplusMultipleScattering = new G4MultipleScattering();
      G4VProcess* theeplusIonisation         = new G4eIonisation();
      G4VProcess* theeplusBremsstrahlung     = new G4eBremsstrahlung();
//      G4VProcess* theeplusIonisation         = new G4LowEnergyIonisation();
//      G4VProcess* theeplusBremsstrahlung     = new G4LowEnergyBremsstrahlung();
      G4VProcess* theeplusAnnihilation       = new G4eplusAnnihilation();
      //
      // add processes
      // (removed)
      //
    }
    else if( particleName == "mu+" || 
               particleName == "mu-"    ) {
    //muon

      G4VProcess* aMultipleScattering = new G4MultipleScattering();
      G4VProcess* aBremsstrahlung     = new G4MuBremsstrahlung();
      G4VProcess* aPairProduction     = new G4MuPairProduction();
      G4VProcess* anIonisation        = new G4MuIonisation();
      
      //
      // add processes
      // (removed)
      //

      //      MulanMaterialStepCut* theMuonStepCut = new MulanMaterialStepCut();
      //      theMuonStepCut->SetHeStep(10*cm);
      //      theMuonStepCut->SetAirStep(10*cm);
      //      theMuonStepCut->SetChamberGasStep(1*mm);
      //      theMuonStepCut->SetVacuumStep(10*cm);
      //      theMuonStepCut->SetMylarStep(50*micrometer);
      //      theMuonStepCut->SetScintStep(50*micrometer);
      //      theMuonStepCut->SetSulfurStep(.1*mm);
      //      theMuonStepCut->SetSilverStep(.025*mm);
      //      theMuonStepCut->SetAlStep(5*nanometer);
      //      theMuonStepCut->SetMaxStep(.8*mm);
      //      pmanager->AddProcess(theMuonStepCut, -1, -1, 5);

    }
    else if (particleName == "proton") {
    //proton
      G4VProcess* protonMultipleScattering = new G4MultipleScattering();
      G4VProcess* protonIonisation         = new G4hIonisation();

      // add processes
      // (removed)
      //
    }
  }
}

#endif  // end of control structure for MulanPhList::ConstructEM()



//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

void MulanPhysicsList::ConstructGeneral()
{

  // Add Decay Process
  G4Decay* theDecayProcess = new G4Decay();
  theParticleIterator->reset();
  while( (*theParticleIterator)() ){
    G4ParticleDefinition* particle = theParticleIterator->value();
    G4ProcessManager* pmanager = particle->GetProcessManager();
    if (theDecayProcess->IsApplicable(*particle)) { 
      pmanager ->AddProcess(theDecayProcess);
      // set ordering for PostStepDoIt and AtRestDoIt
      pmanager ->SetProcessOrdering(theDecayProcess, idxPostStep);
      pmanager ->SetProcessOrdering(theDecayProcess, idxAtRest);
    }
  }

}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

void MulanPhysicsList::SetCuts()
{
  //G4VUserPhysicsList::SetCutsWithDefault method sets 
  //the default cut value for all particle types 
  //
  
  SetCutsWithDefault();
  if (verboseLevel>0)  DumpCutValuesTable();


}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

