#include "MulanVolumeStepCut.hh"

#include "G4Step.hh"
#include "G4UserLimits.hh"
#include "G4VParticleChange.hh"
#include "G4EnergyLossTables.hh"
#include "G4Track.hh"

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....


//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

MulanVolumeStepCut::MulanVolumeStepCut(MulanDetectorConstruction* adetector,
                                       const G4String& aName): 
    	G4VDiscreteProcess(aName),fMaxStep(DBL_MAX),fScaleFactor(DBL_MAX),
    	fTargetStep(DBL_MAX),fHeBagStep(DBL_MAX),fWorldStep(DBL_MAX),
    	fTileStep(DBL_MAX),fSuperTriangleStep(DBL_MAX),fTargetScale(DBL_MAX),
        fHeBagScale(DBL_MAX),fWorldScale(DBL_MAX),fTileScale(DBL_MAX),
        fSuperTriangleScale(DBL_MAX),fBeamLineStep(DBL_MAX),
	fBeamLineScale(DBL_MAX),fBeamWindowStep(DBL_MAX),
	fBeamWindowScale(DBL_MAX)
{
  G4cout<<"MulanVolumeStepCut::MulanVolumeStepCut begun."<<G4endl;
   fTargetLength = adetector->GetTargetLength();
   fHeBagLength = adetector->GetHeBagLength();
   fWorldRadius = adetector->GetWorldRadius();
   fTileLength = adetector->GetTileThickness();
   fSuperTriangleLength = adetector->GetSHLength();
   fBeamLineLength = adetector->GetBeamLineLength();
   fBeamWindowLength = adetector->GetBeamWindowLength();
  G4cout<<"MulanVolumeStepCut::MulanVolumeStepCut ended."<<G4endl;
   if (verboseLevel>0) {
     G4cout << GetProcessName() << " is created "<< G4endl;
   }
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

MulanVolumeStepCut::~MulanVolumeStepCut()
{}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

MulanVolumeStepCut::MulanVolumeStepCut(MulanVolumeStepCut& right)
    :G4VDiscreteProcess(right)
{}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

G4double MulanVolumeStepCut::PostStepGetPhysicalInteractionLength(
                             const G4Track& aTrack,
                             G4double   previousStepSize,
                             G4ForceCondition* condition
                            )
{
  // condition is set to "Not Forced"
  *condition = NotForced;

   G4double ProposedStep = DBL_MAX;
   G4String name = aTrack.GetNextVolume()->GetLogicalVolume()->GetName();

   if (name == "TargetLV") 
   {
    if (fScaleFactor < DBL_MAX) ProposedStep = fScaleFactor*fTargetLength;
    if (fMaxStep < DBL_MAX) ProposedStep = fMaxStep;
    if (fTargetScale < DBL_MAX) ProposedStep = fTargetScale*fTargetLength;
    if (fTargetStep < DBL_MAX) ProposedStep = fTargetStep;
   } 
   else if(name == "BagLV")
   {
    if (fScaleFactor < DBL_MAX) ProposedStep = fScaleFactor*fHeBagLength;
    if (fMaxStep < DBL_MAX) ProposedStep = fMaxStep;
    if (fHeBagScale < DBL_MAX) ProposedStep = fHeBagScale*fHeBagLength;
    if (fHeBagStep < DBL_MAX) ProposedStep = fHeBagStep;
   } 
   else if((name == "WorldLV") || (name == "SphereLV")) 
   {
    if (fScaleFactor < DBL_MAX) ProposedStep = fScaleFactor*fWorldRadius;
    if (fMaxStep < DBL_MAX) ProposedStep = fMaxStep;
    if (fWorldScale < DBL_MAX) ProposedStep = fWorldScale*fWorldRadius;
    if (fWorldStep < DBL_MAX) ProposedStep = fWorldStep;
   } 
   else if((name == "inTileElementLV") || (name == "outTileElementLV"))
   {
    if (fScaleFactor < DBL_MAX) ProposedStep = fScaleFactor*fTileLength;
    if (fMaxStep < DBL_MAX) ProposedStep = fMaxStep;
    if (fTileScale < DBL_MAX) ProposedStep = fTileScale*fTileLength;
    if (fTileStep < DBL_MAX) ProposedStep = fTileStep;
   } 
   else if(name == "SuperTriangleLV")
   {
    if (fScaleFactor < DBL_MAX) ProposedStep = fScaleFactor*fSuperTriangleLength;
    if (fMaxStep < DBL_MAX) ProposedStep = fMaxStep;
    if (fSuperTriangleScale < DBL_MAX) ProposedStep = fSuperTriangleScale*fSuperTriangleLength;
    if (fSuperTriangleStep < DBL_MAX) ProposedStep = fSuperTriangleStep;
   } 
   else if((name == "BeamLineLV") || (name == "WireChamberLV"))
   {
    if (fScaleFactor < DBL_MAX) ProposedStep = fScaleFactor*fBeamLineLength;
    if (fMaxStep < DBL_MAX) ProposedStep = fMaxStep;
    if (fBeamLineScale < DBL_MAX) ProposedStep = fBeamLineScale*fBeamLineLength;
    if (fBeamLineStep < DBL_MAX) ProposedStep = fBeamLineStep;
   } 
   else if((name=="BeamWindowLV") || (name=="ChamberLV"))
   {
    if (fScaleFactor < DBL_MAX) ProposedStep = fScaleFactor*fBeamWindowLength;
    if (fMaxStep < DBL_MAX) ProposedStep = fMaxStep;
    if (fBeamWindowScale < DBL_MAX) ProposedStep = fBeamWindowScale*fBeamWindowLength;
    if (fBeamWindowStep < DBL_MAX) ProposedStep = fBeamWindowStep;
   } 

   return ProposedStep;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

G4VParticleChange* MulanVolumeStepCut::PostStepDoIt(
                             const G4Track& aTrack,
                             const G4Step&
                            )
{
   // do nothing
   aParticleChange.Initialize(aTrack);
   return &aParticleChange;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

inline G4double MulanVolumeStepCut::GetMeanFreePath(const G4Track& aTrack,
                             G4double   previousStepSize,
                             G4ForceCondition* condition
                            )
{
  return 0.;
}

