// This code implementation is the intellectual property of
// the GEANT4 collaboration.
//
// By copying, distributing or modifying the Program (or any work
// based on the Program) you indicate your acceptance of this statement,
// and all its terms.
//
// $Id: BeforeMergingOutputFilesSD.cc,v 1.1.1.1 2003/10/24 19:10:20 asharp Exp $
// GEANT4 tag $Name:  $
//
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

#include "MulanDetectorSD.hh"
#include "MulanRunAction.hh"
#include "MulanPrimaryGeneratorAction.hh"
#include "MulanEventAction.hh"
#include "G4HCofThisEvent.hh"
#include "G4ParticleDefinition.hh"
#include "G4Step.hh"
#include "G4ThreeVector.hh"
#include "G4SDManager.hh"
#include "G4ios.hh"
#include "G4VProcess.hh"
#include "stdio.h"

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

MulanDetectorSD::MulanDetectorSD(G4String name)
:G4VSensitiveDetector(name)
{
  G4String HCname = name;
  collectionName.insert(HCname="detectorCollection");

  //  G4int* index = MulanRunAction::GetRunID();
  char filename1[80];

  // open the io streams and initialize files
  /*
  sprintf (filename1,"/data/20/asharp/Rebuilding/InnerSH.dat");
  InnerSH.open(filename1,ios::out);
  InnerSH<<"TrackID:HexNum:TileNum:Edep:"<<G4endl;

  sprintf (filename1,"/data/20/asharp/Rebuilding/OuterSH.dat");
  OuterSH.open(filename1,ios::out);
  OuterSH<<"TrackID:HexNum:TileNum:Edep:"<<G4endl;

  sprintf (filename1,"/data/20/asharp/Rebuilding/InnerSP.dat");
  InnerSP.open(filename1,ios::out);
  InnerSP<<"TrackID:PentNum:TileNum:Edep:"<<G4endl;

  sprintf (filename1,"/data/20/asharp/Rebuilding/OuterSP.dat");
  OuterSP.open(filename1,ios::out);
  OuterSP<<"TrackID:PentNum:TileNum:Edep:"<<G4endl;

  sprintf (filename1,"/data/20/asharp/Rebuilding/AllTiles.dat");
  AllTiles.open(filename1,ios::out);
  AllTiles<<"SuperNum:TileNum:InSH:OutSH:InSP:OutSP:PosX:PosY:PosZ:Time:LocalTime:"<<G4endl;
  */

  /*
  sprintf(filename1,"/data/20/asharp/Rebuilding/StepsInfo.dat");
  muonInfo.open(filename1,ios::out);
  muonInfo<<"Energy:StopPosX:StopPosY:StopPosZ:StartPosX:StartPosY:StartPosZ:DecayTime:wireFlag:"<<G4endl;

  sprintf(filename1,"/data/20/asharp/Rebuilding/PositronInfo.dat");
  decayInfo.open(filename1,ios::out);
  decayInfo<<"Event:ParticleID:WallSuperID:WallID:Energy:HitSuperID:HitID:HitObject:"<<G4endl;

  // HitObject: 1x for Hex; 2x for pent; x1 = inner tile; x2 = outer tile; x3 = inner lg; x4 = outer lg
 sprintf(filename1,"/data/20/asharp/Rebuilding/TileInfo.dat");
 tileInfo.open(filename1,ios::out);
 tileInfo<<"SuperID:TileID:hitObject:Energy:momX:momY:momZ:depEnergy:Time:"<<G4endl;
  */

 sprintf(filename1,"/data/20/asharp/Rebuilding/MasterInfo.dat");
 masterInfo.open(filename1,ios::out);
 masterInfo<<"SuperID:TileID:hitObject:Energy:momX:momY:momZ:depEnergy:Time:"<<G4endl;

 eventID = 0;
 muonStep = 0;
 eStep = 0;

 for(G4int i = 0;i<10000;i++)
   {
     p_ID[i]=-1;
     S_ID[i]=-1;
     wall_ID[i]=-1;
   }

}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

MulanDetectorSD::~MulanDetectorSD()
{ 
  // destruct all the io streams
  //  InnerSH.close();
  //  OuterSH.close();
  //  InnerSP.close();
  //  OuterSP.close();
  //  muonInfo.close();
  //  decayInfo.close();
  //  tileInfo.close();
  masterInfo.close();
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

void MulanDetectorSD::Initialize(G4HCofThisEvent* HCE)
{
  detectorCollection = new MulanDetectorHitsCollection(SensitiveDetectorName,collectionName[0]); 
  //  cout <<"initializing in SD"<<endl;
  static G4int HCID = -1;
  if(HCID<0)
  { 
    HCID = G4SDManager::GetSDMpointer()->GetCollectionID(collectionName[0]);
  }
  HCE->AddHitsCollection( HCID, detectorCollection ); 

  track = 0.0;
  vertexEnergy = 0.0;
  vertexMom = G4ThreeVector(0.,0.,0.);
  vertexPos = G4ThreeVector(0.,0.,0.);
  InSH_edep = 0;
  OutSH_edep = 0;
  InSP_edep = 0;
  OutSP_edep = 0;
  firstTilePos = G4ThreeVector(0.,0.,0.);
  windowPos   = G4ThreeVector(0.,0.,0.);
  windowMom   = G4ThreeVector(0.,0.,0.);
  chamber1Pos = G4ThreeVector(0.,0.,0.);
  chamber1Mom = G4ThreeVector(0.,0.,0.);
  chamber2Pos = G4ThreeVector(0.,0.,0.);
  chamber2Mom = G4ThreeVector(0.,0.,0.);
  T_ID = -1;
  tileHit =0;
  eventID  +=1;
  badFirstStep = 0;
  energy = 0;
  wall = -1;
  num = -1;
  wireFlag = 0;
  notInTarget = 0;
  NumberOfEvent = 0;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

G4bool MulanDetectorSD::ProcessHits(G4Step* aStep,G4TouchableHistory* ROhist)
{

  G4double edep = aStep->GetTotalEnergyDeposit();
  if(badFirstStep == 1) return false;

  MulanDetectorHit* newHit = new MulanDetectorHit();
  newHit->SetTrackID(aStep->GetTrack()->GetTrackID());
  newHit->SetTileNb(aStep->GetPreStepPoint()->GetTouchable()->GetReplicaNumber());
  newHit->SetEdep(edep);
  newHit->SetPos(aStep->GetPostStepPoint()->GetPosition());
  newHit->SetVertexE(aStep->GetTrack()->GetVertexKineticEnergy());
  detectorCollection->insert(newHit);

  ID = aStep->GetTrack()->GetTrackID();
  G4int parentID = aStep->GetTrack()->GetParentID();

  G4String particleName = aStep->GetTrack()->GetDefinition()->GetParticleName();
  G4VPhysicalVolume* nextVolume = aStep->GetPostStepPoint()->GetPhysicalVolume();
  G4VPhysicalVolume* currentVolume = aStep->GetPreStepPoint()->GetPhysicalVolume();
  const G4VProcess* stepProcess = aStep->GetPostStepPoint()->GetProcessDefinedStep();

  // if the particle at hand is a neutrino, just kill it and move on
  if(!strcmp(particleName,"anti_nu_mu")||!strcmp(particleName,"nu_mu")||
     !strcmp(particleName,"anti_nu_e")||!strcmp(particleName,"nu_e"))
    {
      //      cout <<"killing nu track and returning true"<<endl;
      aStep->GetTrack()->SetTrackStatus((const
					 G4TrackStatus)fKillTrackAndSecondaries);
      return true;
    }

  // we only care about the muon and its decay electron, so kill the rest
  if(parentID != 0 && parentID != 1)
    {
      //      cout <<"killing non-mu or non-e track and returning true"<<endl;
      aStep->GetTrack()->SetTrackStatus((const
					 G4TrackStatus)fKillTrackAndSecondaries);
      return true;
    }

  // if the particle is an e- from the muon ionizing, we aren't
  // interested right now (if you change to mu-, change muonCharge to -1 !)
  G4double muonCharge = 1.;
  G4double daughterCharge = 0.;
  if(!strcmp(particleName,"mu+") || !strcmp(particleName,"mu-"))
    {
      muonCharge = aStep->GetTrack()->GetDefinition()->GetPDGCharge();
    }
  if(parentID == 1)
    {
      daughterCharge = aStep->GetTrack()->GetDefinition()->GetPDGCharge();
      if(daughterCharge != muonCharge)
	{
	  //	  cout <<"killing wrong-charge electron track and returning true"<<endl;
	  aStep->GetTrack()->SetTrackStatus((const
					G4TrackStatus)fKillTrackAndSecondaries);
	  return true;
	}
    }

  // if the particle is a muon, do the following
  if(ID == 1)
    { 
      energy += edep;
      G4String volumeName = currentVolume->GetName();
      if(!strcmp(volumeName,"HorizWire"))
	{
	  if(wireFlag == 2) {wireFlag = 3;}
	  else              {wireFlag = 1;}
	}
      if(!strcmp(volumeName,"VertWire"))
	{
	  if(wireFlag == 1) {wireFlag = 3;}
	  else              {wireFlag = 2;}
	}
      
      // if the muon decays in this step, write out its info
      if(stepProcess && !strcmp(stepProcess->GetProcessName(),"Decay"))
	{
          vertexPos = aStep->GetTrack()->GetVertexPosition();
          vertexMom = aStep->GetTrack()->GetVertexMomentumDirection();
          G4ThreeVector position = aStep->GetPostStepPoint()->GetPosition();
	  localTime = aStep->GetTrack()->GetLocalTime();
	  cout <<"writing to muonInfo"<<endl;
	  //	  muonInfo<<energy<<"  "
	  //		  <<position.x()<<"  "<<position.y()<<"  "<<position.z()<<"  "
	  //		  <<vertexPos.x()<<"  "<<vertexPos.y()<<"  "<<vertexPos.z()<<"  "
	  //		  <<localTime<<"  "<<wireFlag<<G4endl; 
	  masterInfo<<"muon  "<<energy<<"  "
		  <<position.x()<<"  "<<position.y()<<"  "<<position.z()<<"  "
		  <<vertexPos.x()<<"  "<<vertexPos.y()<<"  "<<vertexPos.z()<<"  "
		  <<localTime<<"  "<<wireFlag<<G4endl; 
	}
    }

  G4String volumeName = currentVolume->GetName();

  // now moving on to (+) electrons; wrong-charge electrons should be
  // already killed
  if(parentID == 1 && (!strcmp(volumeName,"InnerTileElement")
		   || !strcmp(volumeName,"InnerSPTileElement")))   
    {
      // values of hit_SuperID, hit_TileID, and hitObject
      hitS_ID = currentVolume->GetMother()->GetCopyNo();
      hitT_ID = currentVolume->GetCopyNo();
      if(!strcmp(volumeName,"InnerTileElement"))   hitObject = 1;
      if(!strcmp(volumeName,"InnerSPTileElement")) hitObject = 2;

      G4double hitEnergy = aStep->GetTrack()->GetKineticEnergy();
      vertexMom = aStep->GetPreStepPoint()->GetMomentum();
      localTime = aStep->GetTrack()->GetLocalTime();

      // write the hit information to the ASCII file
      //      tileInfo<<hitS_ID<<"  "<<hitT_ID<<"  "<<hitObject<<" "<<hitEnergy<<"  "
      //	      <<vertexMom.x()<<"  "<<vertexMom.y()<<"  "<<vertexMom.z()<<"  "
      //	      <<edep<<localTime<<G4endl;
      masterInfo<<"positron  "<<hitS_ID<<"  "<<hitT_ID<<"  "<<hitObject<<" "<<hitEnergy<<"  "
	      <<vertexMom.x()<<"  "<<vertexMom.y()<<"  "<<vertexMom.z()<<"  "
		<<edep<<"  "<<localTime<<G4endl;
      // don't destroy the particle here. we want to know more about it
    }    

  last_ID = ID;
  lastVolumeName = volumeName;   

  const G4VProcess* creatorProcess = aStep->GetTrack()->GetCreatorProcess();

  if(!strcmp(particleName,"e+") && !strcmp(creatorProcess->GetProcessName(),"Decay"))
    {
      if(nextVolume)
	{
	  G4String volumeName = nextVolume->GetName();

	  // if the track is currently in one of these volumes...
	  if(   !strcmp(volumeName,"InnerTileElement")  
	     || !strcmp(volumeName,"OuterTileElement")
	     || !strcmp(volumeName,"InnerSPTileElement")
	     || !strcmp(volumeName,"OuterSPTileElement"))
	    {
	      if(tileHit != 0)
		{
		  vertexPos = aStep->GetTrack()->GetVertexPosition();      
		  vertexMom = aStep->GetTrack()->GetVertexMomentumDirection();
		  vertexEnergy = aStep->GetTrack()->GetVertexKineticEnergy();
		  firstTilePos = aStep->GetPreStepPoint()->GetPosition();
	  // write the decay information to ASCII output file
		  //		  decayInfo<<vertexPos.x()<<"  "<<vertexPos.y()<<"  "<<vertexPos.z()<<"  "
		  //			   <<vertexMom.x()<<"  "<<vertexMom.y()<<"  "<<vertexMom.z()<<"  "
		  //			   <<vertexEnergy<<"  "<<localTime<<"  "
		  //			   <<firstTilePos.x()<<"  "<<firstTilePos.y()<<"  "
		  //			   <<firstTilePos.z()<<G4endl;
		  masterInfo<<"positronvertex  "<<vertexPos.x()<<"  "<<vertexPos.y()<<"  "<<vertexPos.z()<<"  "
			   <<vertexMom.x()<<"  "<<vertexMom.y()<<"  "<<vertexMom.z()<<"  "
			   <<vertexEnergy<<"  "<<localTime<<"  "
			   <<firstTilePos.x()<<"  "<<firstTilePos.y()<<"  "
			   <<firstTilePos.z()<<G4endl;
		}
	      tileHit += 1;
	      //	      cout <<"killing the e+, tileHit was "<<(tileHit-1)<<endl;
	    }
	}
      else  // meaning if (tileHit == 0)
	{
	  //	  cout <<"tileHit was 0, killing"<<endl;
	}  
    }


  if((   !strcmp(volumeName,"InnerTileElement")
      || !strcmp(volumeName,"InnerSPTileElement"))
      && stepProcess != NULL && edep > 0)
    {
      hitT_ID = aStep->GetPreStepPoint()->GetPhysicalVolume()->GetCopyNo();
      hitS_ID = aStep->GetPreStepPoint()->GetPhysicalVolume()->GetMother()->GetCopyNo();
      firstTilePos = aStep->GetPreStepPoint()->GetPosition();
      vertexMom = aStep->GetPreStepPoint()->GetMomentum();
      G4int super;
      if(!strcmp(volumeName,"InnerTileElement")){super = 60;}
      //      else if (!strcmp(volumeName,"OuterTileElement")){super = 61;}
      else if (!strcmp(volumeName,"InnerSPTileElement")){super = 50;}
      //      else if (!strcmp(volumeName,"OuterSPTileElement")){super = 51;}
      else {super = 0;}
      //      cout <<"writing to tileInfo"<<endl;
      /*      tileInfo<<eventID<<"  "<<ID<<"  "<<particleName<<" "
	      <<stepProcess->GetProcessName()<<"  "<<super<<"  "<<hitS_ID<<"  "<<hitT_ID<<"  "
              <<firstTilePos.x()<<"  "<<firstTilePos.y()<<"  "<<firstTilePos.z()<<"  "
              <<vertexMom.x()<<"  "<<vertexMom.y()<<" "<<vertexMom.z()<<"  "<<edep
	      <<G4endl;*/
    }

  if(parentID == 1 && ( !strcmp(volumeName,"InnerTileElement")  
			|| !strcmp(volumeName,"OuterTileElement")
			|| !strcmp(volumeName,"InnerSPTileElement")
			|| !strcmp(volumeName,"OuterSPTileElement")))
  {
    ID      = aStep->GetPreStepPoint()->GetPhysicalVolume()->GetCopyNo();
    G4int houseNo = -1;

    if(aStep->GetPreStepPoint()->GetPhysicalVolume()->GetMother() == NULL)
      { cout <<"Mother volume was NULL"<<endl; }
    else houseNo = aStep->GetPreStepPoint()->GetPhysicalVolume()->GetMother()->GetCopyNo();

    //    cout <<"checking for InnerSH hit"<<endl;
    if(aStep->GetPreStepPoint()->GetPhysicalVolume()->GetName() == "InnerTileElement")
      {
	//	cout <<"writing to InnerSH"<<endl;
	//	InnerSH<<newHit->GetTrackID()<<"  "<<houseNo<<"  "<<ID<<"  "<<newHit->GetEdep()<<G4endl;
	masterInfo<<"IHhit  "<<newHit->GetTrackID()<<"  "<<houseNo<<"  "<<ID<<"  "<<newHit->GetEdep()<<G4endl;
	InSH_edep += newHit->GetEdep();
	time = aStep->GetPreStepPoint()->GetGlobalTime();
	localTime = aStep->GetPreStepPoint()->GetLocalTime();
      }
    //    cout <<"checking for OuterSH hit"<<endl;
    if(aStep->GetPreStepPoint()->GetPhysicalVolume()->GetName() == "OuterTileElement")
      {
	//	cout <<"writing to OuterSH"<<endl;
	//	OuterSH<<newHit->GetTrackID()<<"  "<<houseNo<<"  "<<ID<<"  "<<newHit->GetEdep()<<G4endl;
	masterInfo<<"OHhit  "<<newHit->GetTrackID()<<"  "<<houseNo<<"  "<<ID<<"  "<<newHit->GetEdep()<<G4endl;
	OutSH_edep += newHit->GetEdep();
	time = aStep->GetPreStepPoint()->GetGlobalTime();
	localTime = aStep->GetPreStepPoint()->GetLocalTime();
      }
    //    cout <<"checking for InnerSP hit"<<endl;
    if(aStep->GetPreStepPoint()->GetPhysicalVolume()->GetName() == "InnerSPTileElement")
      {
	//	cout <<"writing to InnerSP"<<endl;
	//	InnerSP<<newHit->GetTrackID()<<"  "<<houseNo<<"  "<<ID<<"  "<<newHit->GetEdep()<<G4endl;
	masterInfo<<"IPhit  "<<newHit->GetTrackID()<<"  "<<houseNo<<"  "<<ID<<"  "<<newHit->GetEdep()<<G4endl;
	InSP_edep += newHit->GetEdep();
	time = aStep->GetPreStepPoint()->GetGlobalTime();
	localTime = aStep->GetPreStepPoint()->GetLocalTime();
      }
    //    cout <<"checking for OuterSP hit"<<endl;
    if(aStep->GetPreStepPoint()->GetPhysicalVolume()->GetName() == "OuterSPTileElement")
      {
	//	cout <<"writing to OuterSP"<<endl;
	//	OuterSP<<newHit->GetTrackID()<<"  "<<houseNo<<"  "<<ID<<"  "<<newHit->GetEdep()<<G4endl;
	masterInfo<<"OPhit  "<<newHit->GetTrackID()<<"  "<<houseNo<<"  "<<ID<<"  "<<newHit->GetEdep()<<G4endl;
	OutSP_edep += newHit->GetEdep();
	time = aStep->GetPreStepPoint()->GetGlobalTime();
	localTime = aStep->GetPreStepPoint()->GetLocalTime();
      }
    //  cout <<"setting vertexPos"<<endl;
  vertexPos = newHit->GetPos();
  }


  //  cout <<"ProcessHits returning true"<<endl;
  return true;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

void MulanDetectorSD::EndOfEvent(G4HCofThisEvent* HCE)
{
  if (verboseLevel>0) {
  //  if (1) {
    G4int NbHits = detectorCollection->entries(); 
    G4cout << "\n-------->Hits Collection: in this event there are " << NbHits 
	   << " hits in the detectors: " << G4endl;
    for (G4int i=0;i<NbHits;i++) (*detectorCollection)[i]->Print();
  }
  
  //Target analysis
  //  outFile1 << distance <<"  "<< vertexEnergy  <<"  "<< vertexMom.x() << "  " 
  //           << vertexMom.y() <<"  " << vertexMom.z() << "  " << vertexPos.x()
  //           << "  " << vertexPos.y() <<"  " << vertexPos.z() << G4endl;
  
  //  outFile2 << vertexMom.x() << "  " << vertexMom.y() <<"  " << vertexMom.z() 
  //           << "  " << vertexPos.x() << "  " << vertexPos.y() <<"  " 
  //           << vertexPos.z() << "  " << edepInner << "  " << edepOuter 
  //           << "  " << firstTilePos.x() << "  " << firstTilePos.y() << "  " 
  //           << firstTilePos.z() << G4endl;
  //   if(ID >-1){AllTiles<<S_ID<<"  "<<ID<<"  "<<InSH_edep<<"  "<<OutSH_edep<<"  "
  //                <<InSP_edep<<"  "<<OutSP_edep<<"  "<<vertexPos.x()<<"  "
  //                <<vertexPos.y()<<"  "<<vertexPos.z()<<"  "<<time<<"  "<<localTime<<G4endl;}

}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....
