// $Id: MulanDetectorSD.backup.cc,v 1.4 2004/03/05 23:33:08 asharp Exp $
// GEANT4 tag $Name:  $
//
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

#include "MulanDetectorSD.hh"
#include "MulanDetectorSD.ecc"
#include "MulanRunAction.hh"
#include "MulanPrimaryGeneratorAction.hh"
#include "MulanEventAction.hh"
#include "G4HCofThisEvent.hh"
#include "G4THitsCollection.hh"
#include "G4ParticleDefinition.hh"
#include "G4Step.hh"
#include "G4ThreeVector.hh"
#include "G4SDManager.hh"
#include "G4ios.hh"
#include "G4VProcess.hh"
#include "stdio.h"
//#include "TObject.h"

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

MulanDetectorSD::MulanDetectorSD(G4String name)
:G4VSensitiveDetector(name)
{
  G4cout<<"MulanDetectorSD instantiating...";
  G4String HCname = name;
  collectionName.insert(HCname="detectorCollection");

  /************* obsolete when the ROOT stuff works ********/
  //  Open the io stream and initialize file
  //  char filename1[80];
  //  sprintf(filename1,"/.autofs/musims/geant4/asharp/new/newInfo.dat");
  //  newInfo.open(filename1,ios::out);
  //  newInfo<<"TrackNumber:SerialNumber:ParticleType:Edep:ParticleEnergy:HitTime:HitPosX:HitPosY:HitPosZ:VtxMomX:VtxMomY:VtxMomZ:VtxEnergy"<<G4endl;
  /************* obsolete when the ROOT stuff works ********/

//    for(G4int i = 0;i<10000;i++) {
//        p_ID[i]=-1;
//        S_ID[i]=-1;
//        wall_ID[i]=-1;
//      }

  /////////////////////////////////////////
  // Declare and initialize ROOT objects //
  /////////////////////////////////////////
  rootFile = new TFile("/.autofs/home/asharp/work/mulan/G4file.root", 
		     "RECREATE");
  //  rootFile = new TFile("/.autofs/home/rmcnabb/mulan/G4/mulan/G4file.root", 
  //		     "RECREATE");
  TotalEdepPerEvent = new TH1D("TotalEdepPerEvent", "Edep in Detector",
			       200, 0., 5.0);
  TotalEdepPerEvent->GetXaxis()->SetTitle("[MeV]");
  NumTilesHit = new TH1D("NumTilesHit", "Number Tile Strikes per Event",
		       21, -0.5, 20.5);
  NumTilesHitEdepCut = new TH1D("NumTilesHitEdepCut", "Number Tile Strikes per Event",
		       21, -0.5, 20.5);
  EdepPerTilePair = new TH2D("EdepPerTilePair", "Edep in Each Tile", 
			     340, 0.5, 340.5, 200, 0, 5);
  EdepPerTilePair->GetXaxis()->SetTitle("Tile Serial Number");
  EdepPerTilePair->GetYaxis()->SetTitle("Edep [MeV]");
  //  G4cerr<<"rootFile has been created"<<G4endl;
  //  TotalEdepPerEvent->SetDirectory(rootFile);

  EvTree = new TTree("tree","MyTree");

  G4cout<<" done"<<G4endl;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

MulanDetectorSD::~MulanDetectorSD()
{ 
  // destruct the io stream
  //  masterInfo.close();

  G4cout<<"This is MulanDetectorSD destructor\n";
  rootFile->Write();
  rootFile->Close();

  //  newInfo<<G4endl;
  //  newInfo.close();
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

void MulanDetectorSD::Initialize(G4HCofThisEvent* HCE)
{
  detectorCollection = new MulanDetectorHitsCollection(SensitiveDetectorName,collectionName[0]); 
  //  G4cerr <<"initializing in SD"<<G4endl;
  static G4int HCID = -1;
  if(HCID<0)
  { 
    HCID = G4SDManager::GetSDMpointer()->GetCollectionID(collectionName[0]);
  }
  HCE->AddHitsCollection( HCID, detectorCollection ); 
  // resetting variables to be used in each step
  badFirstStep = 0;
  T_ID = -1;

  const G4int ARRAY_LENGTH = 50;
  G4double totalEdep = 0;
  G4double singleEdep = 0;
  G4int tilesHit = 0;
  G4int tilesHitECut = 0;
  G4int serialNum;
  G4int partnerSerialNum;
  G4double hitTime = -1.5;
  G4int serialList[ARRAY_LENGTH] = {0};
  G4int numTimesHitList[ARRAY_LENGTH] = {0};
  G4double totalEdepList[ARRAY_LENGTH] = {0};
  G4double hitTimeList[ARRAY_LENGTH] = {0};
  G4int counter = 0;
  for(G4int k=0; k<ARRAY_LENGTH; k++){serialList[k] = -1;}}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

G4bool MulanDetectorSD::ProcessHits(G4Step* aStep,
				    G4TouchableHistory* ROhist)
{
  if(badFirstStep == 1) return false;

  particleName = aStep->GetTrack()->GetDefinition()->GetParticleName();

  // if the particle at hand is a neutrino, just kill it and move on
  if(!strcmp(particleName,"anti_nu_mu")||!strcmp(particleName,"nu_mu")||
     !strcmp(particleName,"anti_nu_e")||!strcmp(particleName,"nu_e")) { 
    //      G4cout <<"killing nu track and returning true"<<G4endl;
    aStep->GetTrack()->SetTrackStatus(
	      (const G4TrackStatus)fKillTrackAndSecondaries);
    return true; 
  }

  parentID = aStep->GetTrack()->GetParentID();
  currentVolume = aStep->GetPreStepPoint()->GetPhysicalVolume();
  volumeName = currentVolume->GetName();
  thisHitTime = aStep->GetTrack()->GetGlobalTime();

  MulanDetectorHit* newHit = new MulanDetectorHit();
  // these numbers we know right away
  newHit->SetTrackNumber(aStep->GetTrack()->GetTrackID());
  newHit->SetEdep(aStep->GetTotalEnergyDeposit());
  newHit->SetParticleMom(aStep->GetPreStepPoint()->GetMomentum());
  newHit->SetHitPosition(aStep->GetPreStepPoint()->GetPosition());
  newHit->SetParticleEnergy(aStep->GetTrack()->GetKineticEnergy());
  newHit->SetVertexMom(aStep->GetTrack()->GetVertexMomentumDirection());
  newHit->SetVertexEnergy(aStep->GetTrack()->GetVertexKineticEnergy());
  newHit->SetHitTime(thisHitTime);

  // calculate the serial number here
  G4int SerialNumber = 0; //hack
  if((!strcmp(volumeName,"InnerTileElement")  
	   || !strcmp(volumeName,"OuterTileElement")
	   || !strcmp(volumeName,"InnerSPTileElement")
	   || !strcmp(volumeName,"OuterSPTileElement"))) {
  if(aStep->GetPreStepPoint()->GetPhysicalVolume()->GetMother() == NULL)
    { G4cerr <<"Mother volume was NULL"<<G4endl; }
  else hitS_ID =
	 aStep->GetPreStepPoint()->GetTouchable()->GetReplicaNumber(1);
  hitT_ID = aStep->GetPreStepPoint()->GetTouchable()->GetReplicaNumber(0);
  const G4int Zipcode = GetZipcode(volumeName,hitS_ID,hitT_ID);
  SerialNumber = GetSerialNumber(Zipcode);
  }
  newHit->SetSerialNumber(SerialNumber);

  //  G4String& creatorProcessName =
  //    aStep->GetTrack()->GetCreatorProcess()->GetProcessName();

  // calculate the particle type number
  G4int type = GetParticleType(particleName,parentID);
  newHit->SetParticleType(type);

  detectorCollection->insert(newHit);

  return true;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

void MulanDetectorSD::EndOfEvent(G4HCofThisEvent* HCE)
{
  ProcessEventHC(HCE);
  //    G4int NbHits = detectorCollection->entries(); 
  //    G4cout << "\n-------->Hits Collection: in this event"
  //                 << there are " << NbHits << " hits in the detectors: "
  //                 << G4endl;
  //    for (G4int i=0;i<NbHits;i++) {
  //	(*detectorCollection)[i]->Print();
  //	(*detectorCollection)[i]->PrintToFile(&newInfo);
  //    }
}

void MulanDetectorSD::ProcessEventHC(G4HCofThisEvent* hitCollections)
{
//    const G4int ARRAY_LENGTH = 50;
//    G4double totalEdep = 0;
//    G4double singleEdep = 0;
//    G4int tilesHit = 0;
//    G4int tilesHitECut = 0;
//    G4int serialNum;
//    G4int partnerSerialNum;
//    G4double hitTime = -1.5;
//    G4int serialList[ARRAY_LENGTH] = {0};
//    G4int numTimesHitList[ARRAY_LENGTH] = {0};
//    G4double totalEdepList[ARRAY_LENGTH] = {0};
//    G4double hitTimeList[ARRAY_LENGTH] = {0};
//    G4int counter = 0;
//    for(G4int k=0; k<ARRAY_LENGTH; k++){serialList[k] = -1;}
  MulanDetectorHitsCollection *aHitCollWithinThisEvent;
  G4int NCollections = hitCollections->GetNumberOfCollections();

  // loop over the Hit Collections (1 for each SD in DetConstruction)
  for(G4int n = 0; n < NCollections; n++) {
    aHitCollWithinThisEvent = (MulanDetectorHitsCollection*) 
      hitCollections->GetHC(n);
    G4int EntriesWithinHitColl = aHitCollWithinThisEvent->entries();

    // loop over the hits within the hit collections
    for(G4int ent = 0; ent < EntriesWithinHitColl; ent++) {
      serialNum = (*aHitCollWithinThisEvent)[ent]->GetSerialNumber();
      hitTime = (*aHitCollWithinThisEvent)[ent]->GetHitTime();
      if(serialNum > 0) {
	singleEdep = (*aHitCollWithinThisEvent)[ent]->GetEdep();
	if(singleEdep>0.125) { tilesHitECut++; }

	counter = 0;
	// search the struck tiles array for same serialNum
	while(serialList[counter]!= serialNum 
	      && serialList[counter] != -1
	      && counter<ARRAY_LENGTH) {counter++;}
	serialList[counter] = serialNum;
	numTimesHitList[counter] += 1;
	totalEdepList[counter] += singleEdep;
	hitTimeList[counter] = hitTime;
	G4cout<<"serialList = ";
	for(G4int i=0; i<counter+1; i++) { 
	  G4cout<<"{"<<serialList[i]<<",";
	  G4cout<<numTimesHitList[i]<<",";
	  G4cout<<totalEdepList[i]<<",";
	  G4cout<<hitTimeList[i]<<"}";
	}
	G4cout<<G4endl;
      } // end of if(serialNum == 0) statement

      else {
	// do something with the non-tile events
      }
      
    } // end of loop over Entries in the hit collection
  } // end of loop over the "multiple" hit collections
  
  //   putting these histogram fills outside the hit 
  //   collections loop is a safety; but not necessary
  //   since we only have one sensitive detector instantiated.
  
  // fill the histograms with data from each hit
  for(G4int c=0; c<ARRAY_LENGTH; c++) {
    totalEdep += totalEdepList[c];
    tilesHit  += numTimesHitList[c];
    EdepPerTilePair->Fill(serialList[c], numTimesHitList[c]);
  }
  TotalEdepPerEvent->Fill(totalEdep);
  TotalEdepPerEvent->Write(TotalEdepPerEvent->GetName(),TObject::kOverwrite);
  NumTilesHit->Fill(tilesHit);
  NumTilesHit->Write(NumTilesHit->GetName(),TObject::kOverwrite);
  NumTilesHitEdepCut->Fill(tilesHitECut);
  NumTilesHitEdepCut->Write(NumTilesHitEdepCut->GetName(),TObject::kOverwrite);
  EdepPerTilePair->Write(EdepPerTilePair->GetName(),TObject::kOverwrite);
}



//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....
G4int MulanDetectorSD::GetParticleType(G4String particleName,
				      G4String creatorProcessName)
{
  if(!strcmp(particleName,"e+") && !strcmp(creatorProcessName,"Decay"))
    {return 1;}
  else if(!strcmp(particleName,"e-"))    {return 2;}
  else if(!strcmp(particleName,"gamma")) {return 3;}
  else if(!strcmp(particleName,"mu+"))   {return 4;}
  else if(!strcmp(particleName,"e+") && strcmp(creatorProcessName,"Decay"))
    {return 5;}
  else {return 6;}
}
G4int MulanDetectorSD::GetParticleType(G4String particleName,
				       G4int parentID)
{
  if(!strcmp(particleName,"e+") && parentID == 1)
    {return 1;}
  else if(!strcmp(particleName,"e-"))    {return 2;}
  else if(!strcmp(particleName,"gamma")) {return 3;}
  else if(!strcmp(particleName,"mu+"))   {return 4;}
  else if(!strcmp(particleName,"e+") && parentID > 1)
    {return 5;}
  else {return 6;}
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....
// Don't use this code - it is for posterity in case I really screw up.
//    // if the particle is a muon, do the following
//    else if((!strcmp(particleName,"mu+") || !strcmp(particleName,"mu-")) && parentID == 0) { 
//      //    G4cerr<<"this is muon loop"<<G4endl;
//      muonTrackingEnergy += edep; // I don't know what to do with this yet
//      // if the muon decays in this step, write out its info
//      if(stepProcessName && !strcmp(stepProcessName,"Decay")) {
//        muonVertexPosition = aStep->GetTrack()->GetVertexPosition();
//        muonVertexMomentum = aStep->GetTrack()->GetVertexMomentumDirection();
//        muonVertexEnergy =   aStep->GetTrack()->GetVertexKineticEnergy();
//        muonDecayPosition = aStep->GetPostStepPoint()->GetPosition();
//        muonDecayTime = aStep->GetTrack()->GetLocalTime();
//        //if(!strcmp(volumeName,"TargetPV")) { DecayedInTarget = true; }
//      }
//    }     // end of muon event processing

//    else if(!strcmp(particleName,"e+") && !RecordedTheElectronVertex &&
//  	  creatorProcessName &&!strcmp(creatorProcessName,"Decay")) {
//      michelVertexEnergy = aStep->GetTrack()->GetVertexKineticEnergy();
//      michelVertexMomentum = aStep->GetPreStepPoint()->GetMomentum();
//      // we don't need to know the creation time. it is recorded w/ muon
    
//      RecordedTheElectronVertex = true; // only need to record this info once
//    }  // end of electron Vertex processing   


//    else if((!strcmp(volumeName,"InnerTileElement")  
//  	   || !strcmp(volumeName,"OuterTileElement")
//  	   || !strcmp(volumeName,"InnerSPTileElement")
//  	   || !strcmp(volumeName,"OuterSPTileElement"))) {
//      // a particle has entered a detector tile
//      //    G4cerr<<"this is a tile hit"<<G4endl;
//      //    if(Energy < 1e-4*eV) { return false; }

//  //   the struck tile is within one of the houses. find which one by
//  //   locating the hit point, accessing its touchable information,
//  //   then looking at the replica number. the argument (1) means to
//  //   go one level up the volume hierarchy tree 
//      // THIS ONLY WORKS ON GEANT4, v. 6.0, not v. 4.0
//  //      if(aStep->GetPreStepPoint()->GetTouchable()->
//  //         GetVolume()->GetMotherLogical()==NULL) {
//  //        G4cerr <<"Mother volume was NULL"<<G4endl; }
//  //      else { hitS_ID = aStep->GetPreStepPoint()->
//  //  	     GetTouchable()->GetReplicaNumber(1); }
//      // the struck tile has its own replica number, hence the (0)
//  //      hitT_ID=aStep->GetPreStepPoint()->GetTouchable()->GetReplicaNumber(0);

//      // this is the way to do it in GEANT4, v. 4.0
//      if(aStep->GetPreStepPoint()->GetPhysicalVolume()->GetMother() == NULL)
//        { G4cerr <<"Mother volume was NULL"<<G4endl; }
//      else {hitS_ID = aStep->GetPreStepPoint()->GetPhysicalVolume()->
//  	    GetMother()->GetCopyNo();}
//      hitT_ID = aStep->GetPreStepPoint()->GetPhysicalVolume()->GetCopyNo();
//      const G4int Zipcode = GetZipcode(volumeName,hitS_ID,hitT_ID);

//      // gather information about this hit
//      if(!hit1Recorded) { // if this is an inner tile
//        hit1Position = aStep->GetPostStepPoint()->GetPosition();
//        hit1ParticleEnergy = aStep->GetTrack()->GetKineticEnergy();
//        hit1EnergyDeposited = newHit->GetEdep();
//        hit1TimeSinceMuonVertex = aStep->GetPreStepPoint()->GetGlobalTime();
//        hit1SerialNumber = GetSerialNumber(Zipcode);
//        G4cerr<<"Calling GetParticleNumber here "<<G4endl;
//        hit1ParticleType = GetParticleType(particleName,creatorProcessName);
//        G4cerr<<"type is "<<hit1ParticleType<<G4endl;
//        hit1Recorded = true;
//        lastVolumeName = volumeName;
//      }
//      else if(hit1Recorded && !strcmp(volumeName,lastVolumeName)
//  	    && hit1SerialNumber==GetSerialNumber(Zipcode)
//  	    && hit1ParticleType==GetParticleType(particleName,creatorProcessName)) {
//        // we are in the same volume as the last hit.
//        // sum the deposited energy
//        hit1EnergyDeposited += newHit->GetEdep();
//      }

//      if(WasInCoincidence) { // a coincidence in this tile already detected
//        WasInCoincidence = false; // reset
//        for(G4int i=0; i<10; i++) {TilesHit[i] = 0;}
//        THcounter = 0;
//      }

//      // particle ID check
//      if(hit1SerialNumber%2 == 0) {
//        // an inner tile
//        TilesHit[THcounter++] = hit1SerialNumber;      
//      }
//      else {
//        // an outer tile
//        for(G4int i=0; i<=THcounter; i++) {
//  	if(TilesHit[i] == (hit1SerialNumber+1)) { WasInCoincidence = true; }
//  	// check for the partner serial number and the same particle type
//        }
//      }

//      if(WasInCoincidence) EventPreparedToPrint = true;
//    } // end of the tile volume if statement
  
//    if (EventPreparedToPrint) {
//      PrintThisHit();
//      EventPreparedToPrint = false;
