// This code implementation is the intellectual property of
// the GEANT4 collaboration.
//
// By copying, distributing or modifying the Program (or any work
// based on the Program) you indicate your acceptance of this statement,
// and all its terms.
//
// $Id: musims_MulanPrimaryGeneratorAction.cc,v 1.1.1.1 2003/10/24 19:10:20 asharp Exp $
// GEANT4 tag $Name:  $
//
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

#include "MulanPrimaryGeneratorAction.hh"
#include "MulanDetectorConstruction.hh"

#include "G4Event.hh"
#include "G4ParticleGun.hh"
#include "G4ParticleTable.hh"
#include "G4ParticleDefinition.hh"
#include "globals.hh"
#include "Randomize.hh"

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

MulanPrimaryGeneratorAction::MulanPrimaryGeneratorAction(MulanDetectorConstruction* myDC,char* inputFile):myDetector(myDC)
{

  //  G4cout << "MulanPrimaryGeneratorAction::MulanPrimaryGeneratorAction begun\n";
  G4int n_particle = 1;
  particleGun = new G4ParticleGun(n_particle);

// default particle
  G4ParticleTable* particleTable = G4ParticleTable::GetParticleTable();
  G4ParticleDefinition* particle = particleTable->FindParticle("mu+");
  
  particleGun->SetParticleDefinition(particle);
  particleGun->SetParticleMomentumDirection(G4ThreeVector(0.,0.,1.));
  //  particleGun->SetParticleEnergy(4.0*MeV);
  particleGun->SetParticleEnergy(3.5*MeV);
  particleGun->SetParticlePolarization(G4ThreeVector(0.,0.,-1.));
  position = -1.0*meter;
  particleGun->SetParticlePosition(G4ThreeVector(0,0,position));
  
  if(inputFile) fInputFile = new ifstream(inputFile,ios::in);
  else fInputFile = NULL;

  //  G4cout <<"MulanPrimaryGeneratorAction::MulanPrimaryGeneratorAction ended\n";
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

MulanPrimaryGeneratorAction::~MulanPrimaryGeneratorAction()
{
  delete particleGun;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

void MulanPrimaryGeneratorAction::GeneratePrimaries(G4Event* anEvent)
{ 
  //  G4cout << "MulanPrimaryGeneratorAction::GeneratePrimaries begun\n";
  G4double r;
  G4double x;
  G4double y;
  G4double xprime;
  G4double yprime;
  G4double momentum;
  G4double crap1,crap2;

  if(fInputFile)
  {
    //read particle info
    *fInputFile >> x >> xprime >> y >> yprime >> momentum >> crap1 >> crap2;

    // transform into Mulan coordinates 
/*
    G4double temp = x;
    x = -y*cm;
    y = temp*cm;
    temp = xprime;
    xprime = -yprime/1000.;
    yprime = temp/1000.;
*/

    position = -75*cm;
    // Obtain momentum components
    x = x*cm;
    y = y*cm;
    xprime = xprime/1000;
    yprime = yprime/1000;
    x = x + position*xprime;
    y = y + position*yprime;    
    momentum = momentum*MeV;


    G4double mass = 105.65836*MeV;
    G4double energy = sqrt(momentum*momentum + mass*mass) - mass;
    G4double xMom = xprime;
    G4double yMom = yprime;
    G4double zMom = sqrt(1-xMom*xMom-yMom*yMom);
    particleGun->SetParticleMomentumDirection(G4ThreeVector(xMom,yMom,zMom));
    particleGun->SetParticleEnergy(energy);
  }
  else
  {
    position = -1.0*meter;
    r = G4UniformRand();
    r = 0*cm;
    x = 0;
    y = 0;

    while(r>2.5*cm)
    {
     x = 5*(G4UniformRand()-0.5)*cm;
     y = 5*(G4UniformRand()-0.5)*cm;
     r = sqrt(x*x+y*y);
    }
  }
  particleGun->SetParticlePolarization(G4ThreeVector(0.,0.,-1.));
  particleGun->SetParticlePosition(G4ThreeVector(x,y,position));  
  particleGun->GeneratePrimaryVertex(anEvent);
  //  G4cout << "MulanPrimaryGeneratorAction::GeneratePrimaries ended\n";
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

