//this is ~/geant4/mulandet/src/MulanDetectorConstruction.cc
#include "MulanDetectorConstruction.hh"
#include "MulanDetectorMessenger.hh"
#include "MulanMagneticField.hh"
#include "MulanDetectorSD.hh"

#include "G4Box.hh"
#include "G4Trd.hh"
#include "G4Trap.hh"
#include "G4Tubs.hh"
#include "G4Sphere.hh"
#include "G4UnionSolid.hh"
#include "G4LogicalVolume.hh"
#include "G4PVPlacement.hh"
#include "G4PVParameterised.hh"
#include "G4SDManager.hh"
#include "G4Polyhedra.hh"
#include "G4Polycone.hh"
#include "G4UserLimits.hh"

#include "G4Assembly.hh"
#include "G4AssemblyCreator.hh"
#include "G4TransportationManager.hh"

#include "G4VisAttributes.hh"
#include "G4Colour.hh"

#include "G4ios.hh"

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

MulanDetectorConstruction::MulanDetectorConstruction()
      :solidWorld(NULL),  logicWorld(NULL),  physiWorld(NULL),
      solidTarget(NULL), logicTarget(NULL), physiTarget(NULL),
      TargetMater(NULL), fpMagField(NULL),
      WorldRadius(0.),  fTargetLength(0.), fHeBagLength(0.)

{
  //  G4cout<<"MulanDetectorConstruction::MulanDetectorConstruction begun."<<G4endl;
   fpMagField = new MulanMagneticField();
   detectorMessenger = new MulanDetectorMessenger(this);
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

MulanDetectorConstruction::~MulanDetectorConstruction()
{
   delete fpMagField;
   delete detectorMessenger;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

  // Look in /usr/local/share/geant4/4.4.0/source/geometry/management/include
/*
void PrintPhysicalVolumeContents(G4PhysicalVolume &physicalvolume)
{
  G4LogicalVolume &logicalvolume = *(physicalvolume.G4LogicalVolume());

  // First print info about me
  cout << "logicalvolume created\n";

  // Now print info about my daughters
  G4int n = logicalvolume.GetNoDaughters();
   for (G4int i=0; i<n; i++)
    {
      G4PhysicalVolume &daughter = *(volume.GetDaughter(i));
      PrintPhysicalVolumeContents(daughter);
    }
}
*/

G4VPhysicalVolume* MulanDetectorConstruction::Construct()
{
  //  G4cout<<"MulanDetectorConstruction::Construct begun."<<G4endl;
   buildMaterials();
   //   G4cout<<"buildMaterials success."<<G4endl;
   //--------- Sizes of the principal geometrical components (solids)  ---------

   //   useful in setting all matter to vacuum
   //   Mylar = Vacuum;
   //   He = Vacuum;
   //   Air = Vacuum;

   hexPsi = pi/6.;  hexTheta = pi/3.;
   pentPsi = pi/5.; pentTheta = 3.*pi/10.;

   fMagField = 0*gauss;

   G4Material* MagnetRingMater = Fe; //to catch muons which traverse magnet ring
   MagnetRingMater = Vacuum; // turn it off

   G4Material* CatcherRingMater = Arnokrome3;
   CatcherRingMater = Vacuum; //turn it off


   //Tile Dimensions
   baseTile = 15. * cm;                 //length of tile base
   thicknessInTile = .3 * cm;           //thickness of inner tile
   thicknessOutTile = thicknessInTile;  //thickness of outer tile
   spaceTile = 0.1 * cm;                //spacing between inner and outer tiles
   InMinusOut = thicknessInTile;        //Diff between tiles along LG edges
   inTileMater = BC404;
   inTileMater = Vacuum;
   outTileMater = inTileMater;
   spacingTile = 1*cm;                  //spacing between adjacent tiles

   //LightGuide Dimensions
   lengthLG = 20.32*cm;           //height or length of LG
   baseLG = baseTile;             //length of LG where scint & LG meet
   topLG = 2.76788*cm;            //size of square where PMT attaches
   thicknessLG = .3175*cm;        //thickness of LG
   offsetLG = 2.2225*cm;          //distance from top of LG to edge of bottom of LG
   LGMater = Mylar;               //for now
   LGMater = Vacuum;               //for now

   //Super Hexagon and Pentagon Dimensions
   //  widthSH = .5*(thicknessInTile+topLG)+.1 * cm; //space between tile and Super hex/pentagon
   widthSH = .635*cm;
   depthSH = 40 * cm;                   //length of Super hex/pentagon house
   baseSH = (baseTile + spacingTile/sin(hexTheta) + 2*widthSH/tan(hexTheta)); //length of SH base
   SHMater = Vacuum;                       //super hex/pentagon house material
   radiusSH = tan(hexTheta)*baseSH/2.;  //radius of circle inscribing SH
   radiusSP = tan(pentTheta)*baseSH/2.; //radius of circle inscribing SP

   //Constants of Icosahedron
   G4double X = .525731112119133606;    // distance from center to
   G4double Z = .850650808352039932;    // apex of triangles assuming unit length
   //  G4double R = sqrt(1./4. - X*X/3.)/X;  //Radius inscribing unit super triangle
   domeRadius = baseSH*sqrt(42.+18.*sqrt(5.))/4.;
   domeSPRadius = sqrt(domeRadius*domeRadius + radiusSH*radiusSH - radiusSP*radiusSP);
   //   G4cout << "dome SPRadius = beamToCenter = " << domeSPRadius << "\n";

   // Actual House Dimensions
   wallThickness = 0.762*mm;
   wallHeight = depthSH - 2*cm;
   wallLengthSH = baseSH - 2*tan(hexPsi)*wallThickness;
   wallLengthSP = baseSH - 2*tan(pentPsi)*wallThickness;
   wallMater = Air;
   wallMater = Vacuum;


   //--------- Set Visualization Attributes -------------------------------------
   G4VisAttributes* whiteSolid = new G4VisAttributes(G4Colour(1,1,1));
   G4VisAttributes* redSolid = new G4VisAttributes(G4Colour(1,0,0));
   G4VisAttributes* greenSolid = new G4VisAttributes(G4Colour(0,1,0));
   G4VisAttributes* blueSolid = new G4VisAttributes(G4Colour(0,0,1));
   G4VisAttributes* magentaSolid = new G4VisAttributes(G4Colour(1,0,1));
   G4VisAttributes* cyanSolid = new G4VisAttributes(G4Colour(0,1,1));
   G4VisAttributes* yellowSolid = new G4VisAttributes(G4Colour(1,1,0));
   G4VisAttributes* whiteFrame = new G4VisAttributes(G4Colour(1,1,1));
   G4VisAttributes* redFrame = new G4VisAttributes(G4Colour(1,0,0));
   G4VisAttributes* greenFrame = new G4VisAttributes(G4Colour(0,1,0));
   G4VisAttributes* blueFrame = new G4VisAttributes(G4Colour(0,0,1));
   G4VisAttributes* magentaFrame = new G4VisAttributes(G4Colour(1,0,1));
   G4VisAttributes* cyanFrame = new G4VisAttributes(G4Colour(0,1,1));
   G4VisAttributes* yellowFrame = new G4VisAttributes(G4Colour(1,1,0));
   G4VisAttributes* offAtt = new G4VisAttributes();
   G4VisAttributes* TargetVisAtt=new G4VisAttributes();
   offAtt->SetVisibility(false);

   whiteSolid->SetForceSolid(true);
   redSolid->SetForceSolid(true);
   greenSolid->SetForceSolid(true);
   blueSolid->SetForceSolid(true);
   magentaSolid->SetForceSolid(true);
   cyanSolid->SetForceSolid(true);
   yellowSolid->SetForceSolid(true);
   whiteFrame->SetForceWireframe(true);
   redFrame->SetForceWireframe(true);
   greenFrame->SetForceWireframe(true);
   blueFrame->SetForceWireframe(true);
   magentaFrame->SetForceWireframe(true);
   cyanFrame->SetForceWireframe(true);
   yellowFrame->SetForceWireframe(true);

   //------------------------------------------------
   // Create Sensitive Detector Manager
   //------------------------------------------------

   G4SDManager* SDman = G4SDManager::GetSDMpointer();

   G4String SDname = "Mulan/SD";
   MulanDetectorSD* aSD = new MulanDetectorSD( SDname );
   SDman->AddNewDetector( aSD );




   //------------------------------
   // World
   //------------------------------
   G4Material* worldMater = Vacuum;

   WorldRadius = 10*(domeRadius+depthSH);
   solidWorld= new G4Sphere("World",0.,WorldRadius,0.,twopi,0.,pi);
   logicWorld= new G4LogicalVolume( solidWorld, worldMater, "WorldLV", 0, 0, 0);
   physiWorld = new G4PVPlacement(0,G4ThreeVector(),"WorldPV",logicWorld,0,false,0);

   logicWorld->SetSensitiveDetector(aSD);  
   logicWorld->SetVisAttributes(offAtt); 

   //------------------------------
   //Sphere enclosing soccer ball
   //------------------------------

   G4Material* sphereMater = Vacuum;
   G4RotationMatrix* sphereRM = new G4RotationMatrix();
   sphereRM->rotateX(0.);
   sphereRM->rotateZ(pi/10.);
   sphereRM->rotateY(-atan(X/Z));
   G4double rMinSphere = domeRadius*.98; //changed back, 6/6/03
   G4double rMaxSphere = (domeRadius+depthSH)*1.1;

   solidSphere = new G4Sphere("Sphere",rMinSphere,rMaxSphere,0.,twopi,0.,pi);
   logicSphere = new G4LogicalVolume(solidSphere, sphereMater,"SphereLV",0,0,0);
   physiSphere = new G4PVPlacement(sphereRM,G4ThreeVector(2000,0,0),"SpherePV",logicSphere,physiWorld,false,0);

   logicSphere->SetSensitiveDetector(aSD);  //Set as sensitive
   logicSphere->SetVisAttributes(blueFrame);
   logicSphere->SetVisAttributes(offAtt);  

   //------------------------------
   // Target
   //------------------------------
   targetSize = 10.*cm;                  //radius of target
   targetThick = 4.5*mm;                 //thickness of target
   TargetMater = Fe;                     //target material

   G4ThreeVector positionTarget = G4ThreeVector(0.,0.,0.);
   solidTarget = new G4Tubs("TargetSolid",0.,targetSize,targetThick/2.,0.,twopi);
   logicTarget = new G4LogicalVolume(solidTarget,TargetMater,"TargetLV",0,0,0);
   physiTarget = new G4PVPlacement(0,positionTarget,"TargetPV",logicTarget,physiWorld,false,0);
   logicTarget->SetSensitiveDetector(aSD);

   //Set Magnetic Field in Target Region
   G4UniformMagField* magField = new G4UniformMagField(G4ThreeVector(0.,fMagField,0));
   G4FieldManager* magManager = new G4FieldManager(magField);
   magManager->CreateChordFinder(magField);
   logicTarget->SetFieldManager(magManager,true);


   //--------------------------------------------------
   // Helium bag (skin is part 1, helium volume part 2)
   //--------------------------------------------------
   G4Material* HeliumBagSkin = Mylar;
   G4Material* HeliumVolume = He;
   G4double HeBagLength = 39*cm; // total length of the helium bag
   G4double HeBagSkinThickness = 30*micrometer;
   G4double HeVolLength = HeBagLength-2*HeBagSkinThickness; // total length of the helium inside
   printf("\n%f %f %f\n\n",HeBagLength,HeBagSkinThickness,HeVolLength);


   // this is the skin of mylar around the helium
   G4double hbz =  targetThick/2 + HeBagLength/2;
   G4ThreeVector positionHeBag = G4ThreeVector(0.,0.,-hbz);
   G4Tubs* solidHeBag = new G4Tubs("HeBagSolid",0.,targetSize, HeBagLength/2.,0.,twopi);
   G4LogicalVolume* logicHeBag = new G4LogicalVolume(solidHeBag, HeliumBagSkin,"HeBagLV",0,0,0);
   G4PVPlacement* physiHeBag = new
     G4PVPlacement(0,positionHeBag,"HeBagLV",logicHeBag,physiWorld,false, 0);

   // this is the helium volume
   G4ThreeVector positionHeVol = G4ThreeVector(0.,0.,0.);
   // this position is relative to the physiHeBag coord system
   G4Tubs* solidHeVol = new G4Tubs("HeVolSolid",0.,targetSize-HeBagSkinThickness, HeVolLength/2.,0.,twopi);
   G4LogicalVolume* logicHeVol = new G4LogicalVolume(solidHeVol,HeliumVolume,"HeVolLV",0,0,0);
   G4PVPlacement* physiHeVol = new G4PVPlacement(0,positionHeVol,"HeVolLV",logicHeVol,physiHeBag,false, 0);


   //------------------------
   // Scattering Volume (Air)
   //------------------------
   G4Material* ScVolMater = Air;
   G4double AirLength = 2.8*cm; // total length of the air

   G4double svz =  targetThick/2 + HeBagLength + AirLength/2;
   G4ThreeVector positionScVol = G4ThreeVector(0.,0.,-svz);
   G4Tubs* solidScVol = new G4Tubs("ScVolSolid",0.,targetSize, AirLength/2.,0.,twopi);
   G4LogicalVolume* logicScVol = new G4LogicalVolume(solidScVol, ScVolMater,"ScVolLV",0,0,0);
   G4PVPlacement* physiScVol = new
     G4PVPlacement(0,positionScVol,"ScVolLV",logicScVol,physiWorld,false, 0);


   //-----------------------------
   // Matter foil to test scattering
   //-----------------------------
   G4Material* FoilMatter = Mylar;

   cout <<"Enter total thickness of foils, in microns: ";
   float dummy;
   fscanf(stdin, "%f", &dummy);
   G4double FoilThickness  = dummy * micrometer;
   printf("\nYou got %f (mm) of foil\n\n", FoilThickness);

   G4double ifz = AirLength/2 - FoilThickness/2;
   G4ThreeVector positionFoil = G4ThreeVector(0.,0.,-ifz);
   G4Tubs* solidFoil = new
     G4Tubs("FoilSolid",0.,targetSize,FoilThickness/2.,0.,twopi);
   G4LogicalVolume* logicFoil = new
     G4LogicalVolume(solidFoil,FoilMatter,"FoilLV",0,0,0);
   G4PVPlacement* physiFoil = new
     G4PVPlacement(0,positionFoil,"FoilLV",logicFoil,physiScVol,false,0);


   //-------------
   // Beam Window
   //-------------
   lengthBeamWindow = 100*micrometer;  // for normal operations
   BeamWindowMater = Mylar;
   //   BeamWindowMater = Vacuum;

   G4double bwz = targetThick/2 + HeBagLength + AirLength + lengthBeamWindow/2;
   G4ThreeVector positionBeamWindow = G4ThreeVector(0.,0.,-bwz);
   G4Tubs* solidBeamWindow = new G4Tubs("BeamWindowSolid",0.,targetSize, lengthBeamWindow/2.,0.,twopi);
   G4LogicalVolume* logicBeamWindow = new G4LogicalVolume(solidBeamWindow, BeamWindowMater,"BeamWindowLV",0,0,0);
   G4PVPlacement* physiBeamWindow = new
     G4PVPlacement(0,positionBeamWindow,"BeamWindowLV",logicBeamWindow,physiWorld,false, 0);

   /**************************************************************/


   //-----------------------------
   // Super Hexagons and Tiles
   //-----------------------------
   G4double rin[2]={0,0};
   G4double rout[2]={radiusSH,radiusSH};
   G4double zPos[2]={0,depthSH};

   solidSH = new G4Polyhedra("SuperHex",hexPsi,twopi,6,2,zPos,rin,rout);

   setHexTileParameters(0);
   solidInTile = new G4Trap("inTileElement",pDz,pTheta,pPhi,pDy1,pDx1,pDx2,pAlpha1,pDy2,pDx3,pDx4,pAlpha2);
   setHexTileParameters(1);
   solidOutTile = new G4Trap("outTileElement",pDz,pTheta,pPhi,pDy1,pDx1,pDx2,pAlpha1,pDy2,pDx3,pDx4,pAlpha2);

   G4Box* solidWallSH = new G4Box("wallSH",wallThickness/2,wallLengthSH/2,wallHeight/2);
   G4LogicalVolume* logicWallSH[20][6];
   G4VPhysicalVolume* physiWallSH[20][6];

   setHexLGParameters(0,0);
   G4Trap* InHexLG1 = new G4Trap("InHexLG1",pDz,pTheta,pPhi,pDy1,pDx1,pDx2,pAlpha1,pDy2,pDx3,pDx4,pAlpha2);
   setHexLGParameters(0,1);
   G4Trap* InHexLG2 = new G4Trap("InHexLG2",pDz,pTheta,pPhi,pDy1,pDx1,pDx2,pAlpha1,pDy2,pDx3,pDx4,pAlpha2);
   G4UnionSolid* InHexLG = new G4UnionSolid("InHexLG",InHexLG1,InHexLG2,hexLGRotate(0),hexLGTranslate(0));

   setHexLGParameters(1,0);
   G4Trap* OutHexLG1 = new G4Trap("OutHexLG1",pDz,pTheta,pPhi,pDy1,pDx1,pDx2,pAlpha1,pDy2,pDx3,pDx4,pAlpha2);
   setHexLGParameters(1,1);
   G4Trap* OutHexLG2 = new G4Trap("OutHexLG2",pDz,pTheta,pPhi,pDy1,pDx1,pDx2,pAlpha1,pDy2,pDx3,pDx4,pAlpha2);
   G4UnionSolid* OutHexLG = new G4UnionSolid("OutHexLG",OutHexLG1,OutHexLG2,hexLGRotate(1),hexLGTranslate(1));

   char nameSH[50];
   for(G4int i = 0; i<20; i++)
   {
      //super hex
      logicSH[i] = new G4LogicalVolume(solidSH,SHMater,"SuperHexLV",0,0,0);
      sprintf(nameSH,"SuperHexPV%d",i);
      physiSH[i] = new G4PVPlacement(hexRM(i),hexPosition(i),nameSH,logicSH[i],physiSphere,true,i);


      for(G4int j = 0; j<6; j++)
      {
         //walls
         logicWallSH[i][j] = new G4LogicalVolume(solidWallSH,wallMater,"hexWallLV",0,0,0);
         physiWallSH[i][j] = new G4PVPlacement(hexWallRM(i,j),hexWallPosition(i,j),"hexWall",logicWallSH[i][j],physiSH[i],true,j);

         //tile elements
         logicInTile[i][j] = new G4LogicalVolume(solidInTile,inTileMater,"inTileElementLV",0,0,0);
         logicOutTile[i][j] = new G4LogicalVolume(solidOutTile,outTileMater,"outTileElementLV",0,0,0);
         logicSHInLG[i][j] = new G4LogicalVolume(InHexLG,LGMater,"inSH_LG_LV",0,0,0);
         logicSHOutLG[i][j] = new G4LogicalVolume(OutHexLG,LGMater,"outSH_LG_LV",0,0,0);

         physiInTile[i][j] = new G4PVPlacement(hexTileRM(i,j),hexTilePosition(i,j),"InnerTileElement",
					       logicInTile[i][j],physiSH[i],true,j);
         physiOutTile[i][j] = new G4PVPlacement(hexTileRM(i,j+6),hexTilePosition(i,j+6),"OuterTileElement",
						logicOutTile[i][j],physiSH[i],true,j);
         physiSHInLG[i][j] = new G4PVPlacement(hexLGRM(i,j),hexLGPosition(i,j),"InnerHexLG",
					       logicSHInLG[i][j],physiSH[i],true,j);
         physiSHOutLG[i][j] = new G4PVPlacement(hexLGRM(i,j+6),hexLGPosition(i,j+6),"OuterHexLG",
						logicSHOutLG[i][j],physiSH[i],true,j);

         //Set Sensitive Detectors
         logicInTile[i][j]->SetSensitiveDetector(aSD);
         logicOutTile[i][j]->SetSensitiveDetector(aSD);
         logicSHInLG[i][j]->SetSensitiveDetector(aSD);
         logicSHOutLG[i][j]->SetSensitiveDetector(aSD);
         logicWallSH[i][j]->SetSensitiveDetector(aSD);

         //Set Vis Properties
	 logicInTile[i][j] ->SetVisAttributes(magentaSolid);
         logicOutTile[i][j]->SetVisAttributes(redSolid); 
         logicSHInLG[i][j]->SetVisAttributes(whiteFrame);  
	 logicSHOutLG[i][j]->SetVisAttributes(whiteFrame); 
	 logicInTile[i][j] ->SetVisAttributes(offAtt);  
	 logicOutTile[i][j]->SetVisAttributes(offAtt);  
	 logicSHInLG[i][j]->SetVisAttributes(offAtt);
	 logicSHOutLG[i][j]->SetVisAttributes(offAtt);
	 logicWallSH[i][j]->SetVisAttributes(offAtt);
      }
      logicSH[i]->SetSensitiveDetector( aSD);   //sets as sensitive detector
      logicSH[i]->SetVisAttributes(offAtt);     // this is really the hex house
   }

   //-----------------------------
   // Super Pentagons and Tiles
   //-----------------------------
   rout[0]=radiusSP;
   rout[1]=radiusSP;
   zPos[0]= 0;
   zPos[1]= depthSH;

   solidSP = new G4Polyhedra("SuperPent",0,twopi,5,2,zPos,rin,rout);

   G4Box* solidWallSP = new G4Box("wallSP",wallThickness/2,wallLengthSP/2,wallHeight/2);
   G4LogicalVolume* logicWallSP[12][5];
   G4VPhysicalVolume* physiWallSP[12][5];

   setPentTileParameters(0);
   solidSPInTile = new G4Trap("inSPTileElement",pDz,pTheta,pPhi,pDy1,pDx1,pDx2,pAlpha1,pDy2,pDx3,pDx4,pAlpha2);

   setPentTileParameters(1);
   solidSPOutTile = new G4Trap("outSPTileElement",pDz,pTheta,pPhi,pDy1,pDx1,pDx2,pAlpha1,pDy2,pDx3,pDx4,pAlpha2);

   setPentLGParameters(0,0);
   G4Trap* InPentLG1 = new G4Trap("InPentLG1",pDz,pTheta,pPhi,pDy1,pDx1,pDx2,pAlpha1,pDy2,pDx3,pDx4,pAlpha2);
   setPentLGParameters(0,1);
   G4Trap* InPentLG2 = new G4Trap("InPentLG2",pDz,pTheta,pPhi,pDy1,pDx1,pDx2,pAlpha1,pDy2,pDx3,pDx4,pAlpha2);

   G4UnionSolid* InPentLG = new G4UnionSolid("InPentLG",InPentLG1,InPentLG2,pentLGRotate(0),pentLGTranslate(0));

   setPentLGParameters(1,0);
   G4Trap* OutPentLG1 = new G4Trap("OutPentLG1",pDz,pTheta,pPhi,pDy1,pDx1,pDx2,pAlpha1,pDy2,pDx3,pDx4,pAlpha2);
   setPentLGParameters(1,1);
   G4Trap* OutPentLG2 = new G4Trap("OutPentLG2",pDz,pTheta,pPhi,pDy1,pDx1,pDx2,pAlpha1,pDy2,pDx3,pDx4,pAlpha2);

   G4UnionSolid* OutPentLG = new G4UnionSolid("OutPentLG",OutPentLG1,OutPentLG2,pentLGRotate(1),pentLGTranslate(1));
   for(i = 1; i<11; i++)
   {
      //super Pentagons
      logicSP[i] = new G4LogicalVolume(solidSP,SHMater,"SuperPentLV",0,0,0);
      physiSP[i] = new G4PVPlacement(pentRM(i),pentPosition(i),"SuperPentPV",logicSP[i],physiSphere,true,i);                // identification
      for(G4int j = 0; j<5; j++)
      {
         //walls
         logicWallSP[i][j] = new G4LogicalVolume(solidWallSP,wallMater,"pentWallLV",0,0,0);
         physiWallSP[i][j] = new G4PVPlacement(pentWallRM(i,j),pentWallPosition(i,j),"pentWall",logicWallSP[i][j],physiSP[i],true,j);

         //Pentagon Tile elements
         logicSPInTile[i][j] = new G4LogicalVolume(solidSPInTile,inTileMater,"inSPTileElementLV",0,0,0);
         logicSPOutTile[i][j] = new G4LogicalVolume(solidSPOutTile,outTileMater,"outSPTileElementLV",0,0,0);
         logicSPInLG[i][j] = new G4LogicalVolume(InPentLG,LGMater,"inSP_LG_LV",0,0,0);
         logicSPOutLG[i][j] = new G4LogicalVolume(OutPentLG,LGMater,"outSP_LG_LV",0,0,0);

         physiSPInTile[i][j] = new G4PVPlacement(pentTileRM(i,j),pentTilePosition(i,j),"InnerSPTileElement",logicSPInTile[i][j],physiSP[i],true,j);
         physiSPOutTile[i][j] = new G4PVPlacement(pentTileRM(i,j+5),pentTilePosition(i,j+5),"OuterSPTileElement",logicSPOutTile[i][j],physiSP[i],true,j);

         physiSPInLG[i][j] = new G4PVPlacement(pentLGRM(i,j),pentLGPosition(i,j),"InnerPentLG",logicSPInLG[i][j],physiSP[i],true,j);
         physiSPOutLG[i][j] = new G4PVPlacement(pentLGRM(i,j+5),pentLGPosition(i,j+5),"OuterPentLG",logicSPOutLG[i][j],physiSP[i],true,j);

      }


   } 




   return physiWorld;
}





//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

void MulanDetectorConstruction::SetTargetMaterial(G4String materialName)
{
   // search the material by its name
   G4Material* pttoMaterial = G4Material::GetMaterial(materialName);
   if (pttoMaterial)
   {
      TargetMater = pttoMaterial;
      logicTarget->SetMaterial(pttoMaterial);
      G4cout << "\n----> The target is " << fTargetLength/cm << " cm of "
      << materialName << G4endl;
   }
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

void MulanDetectorConstruction::SetMagField(G4double fieldValue)
{
   fpMagField->SetFieldValue(fieldValue);
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

void MulanDetectorConstruction::buildMaterials()
{
  //  G4cout<<"MulanDetectorConstruction::buildMaterials begun."<<G4endl;
   //--------- Material definition ---------

   G4double a, iz, z, density;
   G4String name, symbol;
   G4double temperature, pressure;
   G4int nel;

   //elements
   G4double aH = 1.00794*g/mole;
   elH = new G4Element(name="Hydrogen", symbol="H", iz=1, aH);
   G4double aC = 12.0107*g/mole;
   elC = new G4Element(name="Carbon", symbol="C", iz=6, aC);
   G4double aN = 14.01*g/mole;
   elN = new G4Element(name="Nitrogen", symbol="N", iz=7., aN);
   G4double aO = 16.00*g/mole;
   elO = new G4Element(name="Oxygen", symbol="O", iz=8., aO);
   G4double aCl = 35.4527*g/mole;
   elCl = new G4Element("Chlorine", symbol = "Cl", iz = 17., aCl);
   G4double aF = 18.9984032*g/mole;
   elF = new G4Element("Flourine", symbol = "F", iz = 9, aF);
   G4double aS = 32.066*g/mole;
   elS = new G4Element("Sulfur", symbol = "S", iz = 16, aS);
   G4double aCr = 51.9961*g/mole;
   elCr = new G4Element("Chromium", symbol = "Cr",iz = 24,aCr);
   G4double aFe = 55.845*g/mole;
   elFe = new G4Element("Iron", symbol = "Fe" ,iz = 26,aFe);
   G4double aMn = 54.938049*g/mole;
   elMn = new G4Element("Manganese", symbol = "Mn",iz = 25,aMn);
   G4double aNi = 58.6934*g/mole;
   elNi = new G4Element("Nickel", symbol = "Ni",iz = 28,aNi);
   G4double aSi = 28.0855*g/mole;
   elSi = new G4Element("Silicon", symbol = "Si",iz = 14,aSi);
   G4double aW = 183.84*g/mole;
   elW = new G4Element("Tungsten", symbol = "W", iz = 74, aW);
   G4double aAu = 196.967*g/mole;
   elAu = new G4Element("Gold", symbol = "Au", iz = 79, aAu);
   G4double aSn = 118.710*g/mole;
   elSn = new G4Element("Tin", symbol = "Sn", iz = 50, aSn);
   G4double aCo = 58.933*g/mole;
   elCo = new G4Element("Cobalt", symbol = "Co", iz = 27, aCo);

   //Air
   density = 1.2931*mg/cm3;
   Air = new G4Material(name="Air", density, nel=2);
   Air->AddElement(elN, .7);
   Air->AddElement(elO, .3);

   //He = Helium
   a = 4.002602*g/mole;
   density = .1786 *mg/cm3;
   pressure = 1*atmosphere;
   temperature = 293.15*kelvin;  //room temperature, not STP
   He = new G4Material("He",z=2,a,density,kStateGas,temperature,pressure);

   //Al = Aluminum
   a = 26.98*g/mole;
   density = 2.7*g/cm3;
   Aluminium = new G4Material(name="Al", z=13., a, density); 
                                                      
   //Pb = Lead
   a = 207.19*g/mole;
   density = 11.35*g/cm3;
   Pb = new G4Material(name="Pb", z=82., a, density);

   //Ag = Silver
   a = 107.8682*g/mole;
   density = 10.5*g/cm3;
   Ag = new G4Material("Silver", z=47.,a,density);

   //S = Sulfur
   a = 32.006*g/mole;
   //   density = 2.06860*g/cm3;
   density = 1.58*g/cm3;  //this is target made and spec'd by Bernhard
   S = new G4Material("Sulfur", z = 16.,a,density);

   //Fe = Iron
   a = 55.845*g/mole;
   density = 7.874*g/cm3;
   Fe = new G4Material("Iron", z = 26., a, density);

   //W = Tungsten
   a = 183.84*g/mole;
   density = 19.250*g/cm3;
   W = new G4Material("Tungsten", z = 74., a, density);

   //Au = Gold
   a = 196.967*g/mole;
   density = 19.300*g/cm3;
   Au = new G4Material("Gold", z = 79., a, density);

   //Sn = Tin
   a = 118.710*g/mole;
   density = 7.310*g/cm3;
   Sn = new G4Material("Tin", z = 50., a, density);

   //Co = Cobalt
   a = 58.933*g/mole;
   density = 8.90*g/cm3;
   Co = new G4Material("Cobalt", z = 27., a, density);


   //Xenon gas
   density     = 5.858*mg/cm3;
   pressure    = 1*atmosphere;
   temperature = 293.15*kelvin;
   Xenon = new G4Material(name="XenonGas", z=54., a=131.29*g/mole,
                          density, kStateGas,temperature,pressure);

   //Argon gas
   density  = 1.782*mg/cm3;
   pressure = 1*atmosphere;
   temperature = 293.15*kelvin;
   Argon = new G4Material(name="ArgonGas",z=18.,a=39.948*g/mole,density,kStateGas,temperature,pressure);

   //Freon
   density  = 5.418*mg/cm3;
   pressure = 1*atmosphere;
   temperature = 293.15*kelvin;
   Freon = new G4Material("Freon",density,3,kStateGas,temperature,pressure);
   Freon->AddElement(elC, 1);
   Freon->AddElement(elCl, 2);
   Freon->AddElement(elF, 2);

   //Scintillator
   density = 1.032*g/cm3;
   BC404 = new G4Material(name="BC-404", density, nel=2);
   BC408 = new G4Material(name="BC-408", density, nel=2);
   G4double HCRatioBC404 = 1.100;
   G4double HCRatioBC408 = 1.104;
   G4double HWeightBC404 = HCRatioBC404*aH/(HCRatioBC404*aH +aC);
   G4double HWeightBC408 = HCRatioBC408*aH/(HCRatioBC408*aH +aC);
   BC404->AddElement(elH, HWeightBC404);
   BC404->AddElement(elC, 1-HWeightBC404);
   BC408->AddElement(elH, HWeightBC408);
   BC408->AddElement(elC, 1-HWeightBC408);

   //   Vacuum (Galactic)
   density     = 1.e-25*g/cm3;
   pressure    = 1.e-25*atmosphere;
   temperature = pressure/(density*k_Boltzmann);

   //Vacuum (beam)
   //   density     = 1.e-7*mg/cm3;
   //   pressure    = 2.e-2*atmosphere;
   //   temperature = pressure/(density*k_Boltzmann);
   Vacuum = new G4Material(name="Vacuum",density,1,kStateGas, temperature,pressure);
   Vacuum->AddMaterial(Air, 1.);

   //Mylar
   density = 1.39*g/cm3;
   Mylar = new G4Material("Mylar",density,3);
   Mylar->AddElement(elC, 5);
   Mylar->AddElement(elH, 4);
   Mylar->AddElement(elO, 2);

   //Stainless Steel
   density = 7.9*g/cm3;
   SSteel = new G4Material("SSteel",density,5,kStateSolid);
   SSteel->AddElement(elCr,.19);
   SSteel->AddElement(elFe,.6875);
   SSteel->AddElement(elMn,.02);
   SSteel->AddElement(elNi,.0925);
   SSteel->AddElement(elSi,.01);

   //Methane Gas
   density  = 0.6657*mg/cm3;  //look this up
   pressure = 1*atmosphere;  //look this up
   temperature = 293.15*kelvin;
   Methane = new G4Material("Methane",density,2,kStateGas,temperature,pressure);
   Methane->AddElement(elC, 1);
   Methane->AddElement(elH, 4);

   //Wire Chamber Gas Mixture - methane (CH4) plus helium (He)
   density  = 0.6657*mg/cm3;  //look this up
   pressure = 1*atmosphere;   //look this up
   temperature = 293.15*kelvin;
   WireChamberGas = new G4Material("WireChamberGas",density,2,kStateGas,temperature,pressure);
   WireChamberGas->AddMaterial(He, .5);
   WireChamberGas->AddMaterial(Methane, .5);

   //Arnokrome 3 = Magnetizing metal for catcher
   density = 7.6*g/cm3;
   Arnokrome3 = new G4Material("Arnokrome3",density,3,kStateSolid);
   Arnokrome3->AddElement(elCr,.30);
   Arnokrome3->AddElement(elCo,.10);
   Arnokrome3->AddElement(elFe,.60);

   //Arnokrome 4 = Magnetizing metal for catcher
   density = 7.84*g/cm3;
   Arnokrome4 = new G4Material("Arnokrome4",density,2,kStateSolid);
   Arnokrome4->AddElement(elCr,.12);
   Arnokrome4->AddElement(elFe,.88);
   //  G4cout<<"MulanDetectorConstruction::buildMaterials ended."<<G4endl;

}

G4ThreeVector MulanDetectorConstruction::getVertex(G4int vertex)
{
   G4double X = .525731112119133606; // constant of super triangle
   G4double Z = .850650808352039932; // other constant of super triangle
   G4ThreeVector vdata[12] = {
                                G4ThreeVector(-X, 0.0, Z),G4ThreeVector(X, 0.0, Z),
                                G4ThreeVector(-X, 0.0, -Z),G4ThreeVector(X, 0.0, -Z),
                                G4ThreeVector(0.0, Z, X),G4ThreeVector(0.0, Z, -X),
                                G4ThreeVector(0.0, -Z, X),G4ThreeVector(0.0, -Z, -X),
                                G4ThreeVector(Z, X, 0.0),G4ThreeVector(-Z, X, 0.0),
                                G4ThreeVector(Z, -X, 0.0),G4ThreeVector(-Z, -X, 0.0)};
   return vdata[vertex];

}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

G4int MulanDetectorConstruction::getHexIndex(G4int hex,G4int vertex)
{
   G4int indices[20][3] = {
                             {0,4,1}, {0,1,6}, {0,6,11}, {0,11,9}, {0,9,4},
                             {5,4,9}, {2,9,11}, {7,11,6}, {10,6,1}, {8,1,4},
                             {11,7,2}, {9,2,5}, {4,5,8}, {1,8,10}, {6,10,7},
                             {3,7,10}, {3,10,8}, {3,8,5}, {3,5,2}, {3,2,7} };
   return indices[hex][vertex];
}


G4ThreeVector MulanDetectorConstruction::hexPosition(G4int hex)
{
   G4ThreeVector zAxis;
   G4ThreeVector position;
   G4int index[3] = {getHexIndex(hex,0),getHexIndex(hex,1),getHexIndex(hex,2)};
   zAxis = (getVertex(index[0]) + getVertex(index[1]) + getVertex(index[2]))*(1./3.);
   zAxis = zAxis.unit();
   position = zAxis*domeRadius;
   return position;
}

G4RotationMatrix* MulanDetectorConstruction::hexRM(G4int hex)
{
   G4ThreeVector xAxis;
   G4ThreeVector yAxis;
   G4ThreeVector zAxis;
   G4RotationMatrix* RM = new G4RotationMatrix();
   G4int index[3] = {getHexIndex(hex,0),getHexIndex(hex,1),getHexIndex(hex,2)};
   zAxis = (getVertex(index[0]) + getVertex(index[1]) + getVertex(index[2]))*(1./3.);
   xAxis = getVertex(index[0]) - zAxis;
   yAxis = zAxis.cross(xAxis);

   xAxis = xAxis.unit();
   yAxis = yAxis.unit();
   zAxis = zAxis.unit();

   G4ThreeVector newX(xAxis.x(),yAxis.x(),zAxis.x());
   G4ThreeVector newY(xAxis.y(),yAxis.y(),zAxis.y());
   G4ThreeVector newZ(xAxis.z(),yAxis.z(),zAxis.z());

   RM->rotateAxes(newX, newY, newZ);
   return RM;
}

void MulanDetectorConstruction::setHexTileParameters(int tile) //tile = (0 => inner, 1=>outer)
{
   pDz = thicknessInTile/2.;
   pTheta = 0.;
   pPhi = 0.;
   pDy1 = .25*baseTile/tan(hexPsi);
   pDx1 = DBL_MIN;
   pDx2 = baseTile/2.;
   pDy2 = pDy1;
   pDx3 = pDx1;
   pDx4 = pDx2;
   pAlpha1 = 0.;
   pAlpha2 = pAlpha1;
   if(tile ==1)
   {
      pPhi = getHexTileBevel();
      pTheta = 0;
      pDz = thicknessOutTile/2.;
      pDy2 = (baseTile*cos(hexPsi) - 2*InMinusOut)/(4*sin(hexPsi));
      pDy1 = pDy2 - thicknessOutTile*tan(pPhi);
      pDx4 = baseTile/2. - InMinusOut/cos(hexPsi);
      pDx2 = pDy1*pDx4/pDy2;
      //  pDx2 = pDx4 - 2*thicknessOutTile*tan(pPhi)*tan(hexPsi);
      pDx1 = DBL_MIN;
      pDx3 = pDx1;
      pAlpha1 = 0;
      pAlpha2 = 0;
   }
}

G4ThreeVector MulanDetectorConstruction::hexTilePosition(G4int hex, G4int tile)
{
   G4double RMax = .25*baseTile/tan(hexPsi) + .5*spacingTile/sin(hexPsi);
   G4double tileX;
   G4double tileY;
   G4ThreeVector tilePlacement;

   if(hex < 10)
   {
      if(tile < 6)
      {
         tileX = RMax*cos(twopi*tile/6.);
         tileY = RMax*sin(twopi*tile/6.);
         tilePlacement = G4ThreeVector(tileX,tileY,thicknessInTile/2.);
      }
      else
      {
         setHexTileParameters(1);
         tileX = (radiusSH-widthSH-(pDy2-pDz*tan(pPhi)))*cos(twopi*tile/6.);
         tileY = (radiusSH-widthSH-(pDy2-pDz*tan(pPhi)))*sin(twopi*tile/6.);
         tilePlacement = G4ThreeVector(tileX,tileY,1*thicknessInTile+.5*thicknessOutTile+spaceTile);
      }
   }
   else
   {
      if(tile < 6)
      {
         tileX = RMax*cos(-twopi*tile/6.);
         tileY = RMax*sin(-twopi*tile/6.);
         tilePlacement = G4ThreeVector(tileX,tileY,thicknessInTile/2.);
      }
      else
      {
         setHexTileParameters(1);
         tileX = (radiusSH-widthSH-(pDy2-pDz*tan(pPhi)))*cos(-twopi*tile/6.);
         tileY = (radiusSH-widthSH-(pDy2-pDz*tan(pPhi)))*sin(-twopi*tile/6.);
         tilePlacement = G4ThreeVector(tileX,tileY,1*thicknessInTile+.5*thicknessOutTile+spaceTile);
      }
   }
   return tilePlacement;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

G4RotationMatrix* MulanDetectorConstruction::hexWallRM(G4int hex,G4int wall)
{
   G4RotationMatrix* RM = new G4RotationMatrix();
   if(hex<10)
   {
      RM->rotateZ((-wall)*twopi/6.);
   }
   else
   {
      RM->rotateZ((+wall)*twopi/6.);
   }
   return RM;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

G4ThreeVector MulanDetectorConstruction::hexWallPosition(G4int hex,G4int wall)
{
   G4double RMax = radiusSH - wallThickness/2;
   G4double X;
   G4double Y;
   G4ThreeVector Placement;
   if(hex<10)
   {
      X = RMax*cos(twopi*wall/6.);
      Y = RMax*sin(twopi*wall/6.);
   }
   else
   {
      X = RMax*cos(-twopi*wall/6.);
      Y = RMax*sin(-twopi*wall/6.);
   }
   Placement = G4ThreeVector(X,Y,wallHeight/2);
   return Placement;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

G4RotationMatrix* MulanDetectorConstruction::hexTileRM(G4int hex, G4int tile)
{
   G4RotationMatrix* RM = new G4RotationMatrix();
   if(hex < 10)
   {
      RM->rotateZ((1.5-tile)*twopi/6.);
   }
   else
   {
      RM->rotateZ((1.5+tile)*twopi/6.);
   }
   return RM;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

G4double MulanDetectorConstruction::getHexTileBevel()
{
   return atan((radiusSH - widthSH)/(2*(domeRadius+thicknessInTile+spaceTile+thicknessOutTile)));
}

void MulanDetectorConstruction::setHexLGParameters(int tile, int part) //tile = (0=>inner, 1=>outer)
//part = (0=>lower, 1=>upper)
{
   if(tile==0)
   {
      setHexTileParameters(0);
      pDz = thicknessLG/2.;
      pDy1 = .5*sqrt((4*pDy2*pDy2)+(pDx4*pDx4));
      pDy2 = pDy1;
      pDx1 = DBL_MIN;
      pDx2 = pDx1;
      pDx3 = thicknessLG/2.;
      pDx4 = pDx3;
      pTheta = atan(.5);
      pPhi = pi;
      pAlpha1 = 0;
      pAlpha2 = 0;
      if(part==1)
      {
         pDz = lengthLG/2.;
         pDy1 = pDy1;
         pDy2 = 3*thicknessLG;
         pDx1 = pDx3;
         pDx2 = pDx1;
         pDx3 = topLG/2.;
         pDx4 = pDx3;
         pTheta = atan(offsetLG/(2*pDz));
         pPhi = 0;
         pAlpha1 = 0;
         pAlpha2 = 0;
      }
   }
   else
   {
      setHexTileParameters(1);
      pDz = thicknessLG/2.;
      pDy1 = .5*sqrt((4*pDy2*pDy2)+(pDx4*pDx4));
      pDy2 = pDy1;
      pDx1 = DBL_MIN;
      pDx2 = pDx1;
      pDx3 = thicknessLG/2.;
      pDx4 = pDx3;
      pTheta = atan(.5);
      pPhi = pi;
      pAlpha1 = 0;
      pAlpha2 = 0;
      if(part==1)
      {
         pDz = lengthLG/2. - thicknessInTile;
         pDy1 = pDy1;
         pDy2 = 3*thicknessLG;
         pDx1 = pDx3;
         pDx2 = pDx1;
         pDx3 = topLG/2.;
         pDx4 = pDx3;
         pTheta = atan(offsetLG/(2*pDz));
         pPhi = 0;
         pAlpha1 = 0;
         pAlpha2 = 0;
      }
   }

}

G4ThreeVector MulanDetectorConstruction::hexLGTranslate(int tile)
{
   G4ThreeVector translate;
   setHexLGParameters(tile,1);
   if(tile == 0)
   {
      translate = G4ThreeVector(.5*(offsetLG - .5*thicknessLG),0,pDz+thicknessLG/2 +.005);
   }
   else
   {
      translate = G4ThreeVector(.5*(offsetLG - .5*thicknessLG),0,pDz+thicknessLG/2+.005);
   }
   return translate;
}

G4RotationMatrix* MulanDetectorConstruction::hexLGRotate(int tile)
{
   G4RotationMatrix* RM = new G4RotationMatrix();
   return RM;
}

G4RotationMatrix* MulanDetectorConstruction::hexLGRM(G4int hex,G4int tile)
{
   G4RotationMatrix* RM = new G4RotationMatrix();
   if(hex < 10)
   {
      if(tile<6)
      {
         RM->rotateZ(2*hexPsi - tile*twopi/6.);
      }
      else
      {
         RM->rotateZ(-2*hexPsi - tile*twopi/6.);
      }
   }
   else
   {
      if(tile<6)
      {
         RM->rotateZ(2*hexPsi + tile*twopi/6.);
      }
      else
      {
         RM->rotateZ(-2*hexPsi + tile*twopi/6.);
      }
   }
   return RM;
}

G4ThreeVector MulanDetectorConstruction::hexLGPosition(G4int hex, G4int tile)
{
   G4ThreeVector Placement;
   G4double Rmax;
   G4double phi;
   G4double deltaX;
   G4double deltaY;
   G4int index;

   if(tile<6)
      index = 0;
   else
      index = 1;
   setHexTileParameters(index);
   Placement = hexTilePosition(0,tile);
   Rmax = sqrt(Placement.x()*Placement.x() + Placement.y()*Placement.y());
   deltaX = pDy2*tan(hexPsi)+.25*thicknessLG*cos(hexPsi);
   deltaY = Rmax - .25*thicknessLG*sin(hexPsi);
   phi = atan(deltaX/deltaY);
   Rmax = sqrt(deltaX*deltaX + deltaY*deltaY);

   if(hex < 10)
   {
      if(tile < 6)
      {
         deltaX = Rmax*cos(phi + twopi*tile/6);
         deltaY = Rmax*sin(phi + twopi*tile/6);
         Placement = G4ThreeVector(deltaX,deltaY,thicknessLG/2.);
      }
      else
      {
         deltaX = Rmax*cos(-phi + twopi*tile/6);
         deltaY = Rmax*sin(-phi + twopi*tile/6);
         Placement = G4ThreeVector(deltaX,deltaY,thicknessInTile+spaceTile+thicknessLG/2.);
      }
   }
   else
   {
      if(tile < 6)
      {
         deltaX = Rmax*cos(phi - twopi*tile/6);
         deltaY = Rmax*sin(phi - twopi*tile/6);
         Placement = G4ThreeVector(deltaX,deltaY,thicknessLG/2.);
      }
      else
      {
         deltaX = Rmax*cos(-phi - twopi*tile/6);
         deltaY = Rmax*sin(-phi - twopi*tile/6);
         Placement = G4ThreeVector(deltaX,deltaY,thicknessInTile+spaceTile+thicknessLG/2.);
      }
   }

   return Placement;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

G4int MulanDetectorConstruction::getPentIndex(G4int pent,G4int vertex)
{
   G4int indices[12][2] = {
                             {0,11}, {11,9}, {9,4},
                             {4,1}, {1,6}, {6,11},
                             {5,2}, {2,7}, {7,10},
                             {10,8}, {8,5}, {3,5} };
   return indices[pent][vertex];
}


G4ThreeVector MulanDetectorConstruction::pentPosition(G4int pent)
{
   G4ThreeVector zAxis;
   G4ThreeVector position;
   G4int index[2] = {getPentIndex(pent,0),getPentIndex(pent,1)};
   zAxis = getVertex(index[0]);
   zAxis = zAxis.unit();
   position = zAxis*domeSPRadius;
   return position;
}

G4RotationMatrix* MulanDetectorConstruction::pentRM(G4int pent)
{
   G4ThreeVector xAxis;
   G4ThreeVector yAxis;
   G4ThreeVector zAxis;
   G4RotationMatrix* RM = new G4RotationMatrix();
   G4int index[2] = {getPentIndex(pent,0),getPentIndex(pent,1)};
   zAxis = getVertex(index[0]);
   yAxis = zAxis.cross(getVertex(index[1]));
   xAxis = yAxis.cross(zAxis);

   xAxis = xAxis.unit();
   yAxis = yAxis.unit();
   zAxis = zAxis.unit();

   G4ThreeVector newX(xAxis.x(),yAxis.x(),zAxis.x());
   G4ThreeVector newY(xAxis.y(),yAxis.y(),zAxis.y());
   G4ThreeVector newZ(xAxis.z(),yAxis.z(),zAxis.z());

   RM->rotateAxes(newX, newY, newZ);
   return RM;
}

void MulanDetectorConstruction::setPentTileParameters(int tile)
{
   pDz = thicknessInTile/2.;
   pTheta = 0.;
   pPhi = 0.;
   pDy1 = .25*baseTile/tan(pentPsi);
   pDx1 = DBL_MIN;
   pDx2 = baseTile/2.;
   pDy2 = pDy1;
   pDx3 = pDx1;
   pDx4 = pDx2;
   pAlpha1 = 0.;
   pAlpha2 = pAlpha1;
   if(tile == 1)
   {
      pPhi = getPentTileBevel();
      pTheta = 0;
      pDz = thicknessOutTile/2.;
      pDy2 = (baseTile*cos(pentPsi) - 2*InMinusOut)/(4*sin(pentPsi));
      pDy1 = pDy2 - thicknessOutTile*tan(pPhi);
      pDx4 = baseTile/2. - InMinusOut/cos(pentPsi);
      pDx2 = pDy1*pDx4/pDy2;
      pDx1 = DBL_MIN;
      pDx3 = pDx1;
      pAlpha1 = 0;
      pAlpha2 = 0;

   }
}

G4RotationMatrix* MulanDetectorConstruction::pentWallRM(G4int pent,G4int wall)
{
   G4RotationMatrix* RM = new G4RotationMatrix();
   if(pent<6)
   {
      RM->rotateZ(-pentPsi-wall*twopi/5.);
   }
   else
   {
      RM->rotateZ(-pentPsi+wall*twopi/5.);
   }

   return RM;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

G4ThreeVector MulanDetectorConstruction::pentWallPosition(G4int pent,G4int wall)
{
   G4double RMax = radiusSP - wallThickness/2;
   G4double X;
   G4double Y;
   G4ThreeVector Placement;
   if(pent<6)
   {
      X = RMax*cos(pentPsi+twopi*wall/5.);
      Y = RMax*sin(pentPsi+twopi*wall/5.);
   }
   else
   {
      X = RMax*cos(pentPsi-twopi*wall/5.);
      Y = RMax*sin(pentPsi-twopi*wall/5.);
   }
   Placement = G4ThreeVector(X,Y,wallHeight/2);
   return Placement;
}


G4ThreeVector MulanDetectorConstruction::pentTilePosition(G4int pent, G4int tile)
{
   G4double RMax = .25*baseTile/tan(pentPsi) + .5*spacingTile/sin(pentPsi);
   G4double tileX;
   G4double tileY;
   G4ThreeVector tilePlacement;

   if(pent<6)
   {
      if(tile<5)
      {
         tileX = RMax*cos(pentPsi+twopi*tile/5.);
         tileY = RMax*sin(pentPsi+twopi*tile/5.);
         tilePlacement = G4ThreeVector(tileX,tileY,thicknessInTile/2.);
      }
      if(tile>4)
      {
         setPentTileParameters(1);
         tileX = (radiusSP-widthSH-(pDy2-pDz*tan(pPhi)))*cos(pentPsi+twopi*tile/5.);
         tileY = (radiusSP-widthSH-(pDy2-pDz*tan(pPhi)))*sin(pentPsi+twopi*tile/5.);
         tilePlacement = G4ThreeVector(tileX,tileY,thicknessInTile+.5*thicknessOutTile+spaceTile);
      }
   }

   if(pent>5)
   {
      if(tile<5)
      {
         tileX = RMax*cos(pentPsi-twopi*tile/5.);
         tileY = RMax*sin(pentPsi-twopi*tile/5.);
         tilePlacement = G4ThreeVector(tileX,tileY,thicknessInTile/2.);
      }
      if(tile>4)
      {
         setPentTileParameters(1);
         tileX = (radiusSP-widthSH-(pDy2-pDz*tan(pPhi)))*cos(pentPsi-twopi*tile/5.);
         tileY = (radiusSP-widthSH-(pDy2-pDz*tan(pPhi)))*sin(pentPsi-twopi*tile/5.);
         tilePlacement = G4ThreeVector(tileX,tileY,thicknessInTile+.5*thicknessOutTile+spaceTile);
      }
   }
   return tilePlacement;
}
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo.....

G4RotationMatrix* MulanDetectorConstruction::pentTileRM(G4int pent, G4int tile)
{

   G4RotationMatrix* RM = new G4RotationMatrix();
   if(pent < 6)
   {
      RM->rotateZ(-pentPsi+(1.25-tile)*twopi/5.);
   }
   else
   {
      RM->rotateZ(-pentPsi+(1.25+tile)*twopi/5.);
   }
   return RM;
}

G4double MulanDetectorConstruction::getPentTileBevel()
{
   return atan((radiusSP - widthSH)/(2*(domeSPRadius+thicknessInTile+spaceTile+thicknessOutTile)));
}

void MulanDetectorConstruction::setPentLGParameters(int tile, int part) //tile = (0=>inner, 1=>outer)
//part = (0=>lower, 1=>upper)
{
   if(tile==0)
   {
      setPentTileParameters(0);
      pDz = thicknessLG/2.;
      pDy1 = .5*sqrt((4*pDy2*pDy2)+(pDx4*pDx4));
      pDy2 = pDy1;
      pDx1 = DBL_MIN;
      pDx2 = pDx1;
      pDx3 = thicknessLG/2.;
      pDx4 = pDx3;
      pTheta = atan(.5);
      pPhi = pi;
      pAlpha1 = 0;
      pAlpha2 = 0;
      if(part==1)
      {
         pDz = lengthLG/2.;
         pDy1 = pDy1;
         pDy2 = 3*thicknessLG;
         pDx1 = pDx3;
         pDx2 = pDx1;
         pDx3 = topLG/2.;
         pDx4 = pDx3;
         pTheta = atan(offsetLG/(2*pDz));
         pPhi = 0;
         pAlpha1 = 0;
         pAlpha2 = 0;
      }
   }
   else
   {
      setPentTileParameters(1);
      pDz = thicknessLG/2.;
      pDy1 = .5*sqrt((4*pDy2*pDy2)+(pDx4*pDx4));
      pDy2 = pDy1;
      pDx1 = DBL_MIN;
      pDx2 = pDx1;
      pDx3 = thicknessLG/2.;
      pDx4 = pDx3;
      pTheta = atan(.5);
      pPhi = pi;
      pAlpha1 = 0;
      pAlpha2 = 0;
      if(part==1)
      {
         pDz = lengthLG/2.- thicknessInTile;
         pDy1 = pDy1;
         pDy2 = 3*thicknessLG;
         pDx1 = pDx3;
         pDx2 = pDx1;
         pDx3 = topLG/2.;
         pDx4 = pDx3;
         pTheta = atan(offsetLG/(2*pDz));
         pPhi = 0;
         pAlpha1 = 0;
         pAlpha2 = 0;
      }
   }
}

G4ThreeVector MulanDetectorConstruction::pentLGTranslate(int tile)
{
   G4ThreeVector translate;
   setPentLGParameters(tile,1);
   if(tile == 0)
   {
      translate = G4ThreeVector(.5*(offsetLG - .5*thicknessLG),0,pDz+thicknessLG/2+.005);
   }
   else
   {
      translate = G4ThreeVector(.5*(offsetLG - .5*thicknessLG),0,pDz+thicknessLG/2+.005);
   }
   return translate;
}

G4RotationMatrix* MulanDetectorConstruction::pentLGRotate(int tile)
{
   G4RotationMatrix* RM = new G4RotationMatrix();
   return RM;
}

G4RotationMatrix* MulanDetectorConstruction::pentLGRM(G4int pent,G4int tile)
{
   G4RotationMatrix* RM = new G4RotationMatrix();
   if(pent < 6)
   {
      if(tile<5)
      {
         RM->rotateZ(pentPsi/2 - tile*twopi/5.);
      }
      else
      {
         RM->rotateZ((-1.25- tile)*twopi/5.);
      }
   }
   else
   {
      if(tile<5)
      {
         RM->rotateZ(pentPsi/2+tile*twopi/5.);
      }
      else
      {
         RM->rotateZ((-1.25 + tile)*twopi/5.);
      }
   }
   return RM;
}

G4ThreeVector MulanDetectorConstruction::pentLGPosition(G4int pent, G4int tile)
{
   G4ThreeVector Placement;
   G4double Rmax;
   G4double phi;
   G4double deltaX;
   G4double deltaY;
   G4int index;

   if(tile<5)
      index = 0;
   else
      index = 1;
   setPentTileParameters(index);
   Placement =pentTilePosition(0,tile);
   Rmax = sqrt(Placement.x()*Placement.x() + Placement.y()*Placement.y());
   deltaX = pDy2*tan(pentPsi)+.25*thicknessLG*cos(pentPsi);
   deltaY = Rmax - .25*thicknessLG*sin(pentPsi);
   phi = atan(deltaX/deltaY);
   Rmax = sqrt(deltaX*deltaX + deltaY*deltaY);

   if(pent < 6)
   {
      if(tile < 5)
      {
         deltaX = Rmax*cos(pentPsi + phi + twopi*tile/5);
         deltaY = Rmax*sin(pentPsi + phi + twopi*tile/5);
         Placement = G4ThreeVector(deltaX,deltaY,thicknessLG/2.);
      }
      else
      {
         deltaX = Rmax*cos(pentPsi -phi + twopi*tile/5);
         deltaY = Rmax*sin(pentPsi -phi + twopi*tile/5);
         Placement = G4ThreeVector(deltaX,deltaY,thicknessInTile+spaceTile+thicknessLG/2.);
      }
   }
   else
   {
      if(tile < 5)
      {
         deltaX = Rmax*cos(pentPsi + phi - twopi*tile/5);
         deltaY = Rmax*sin(pentPsi + phi - twopi*tile/5);
         Placement = G4ThreeVector(deltaX,deltaY,thicknessLG/2.);
      }
      else
      {
         deltaX = Rmax*cos(pentPsi -phi - twopi*tile/5);
         deltaY = Rmax*sin(pentPsi -phi - twopi*tile/5);
         Placement = G4ThreeVector(deltaX,deltaY,thicknessInTile+spaceTile+thicknessLG/2.);
      }
   }

   return Placement;
}
